# -*- coding: utf-8 -*-
##############################################################################
#
#    Module Chemicals CLP for Chemical Industries (OpenERP)
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

from osv import fields, osv
import pooler
from tools.translate import _



class clp_hazard_type(osv.osv):
    _name = "clp.hazard.type"
    _description = 'Type of hazard'
    
    _columns = {
        'name': fields.char(size=64, string='Name', translate=True, required=True),
    }
    _order = 'id'

clp_hazard_type()

class clp_hazard_class(osv.osv):
    _name = "clp.hazard.class"
    _description = 'Class of hazard'
    
    _columns = {
        'name': fields.char(size=128, string='Class', translate=True, required=True),
        'type_id': fields.many2one('clp.hazard.type', 'Type of hazard', required=True),
    }
    _order = 'type_id, id'

clp_hazard_class()

class clp_hazard_class_category(osv.osv):
    _name = "clp.hazard.class.category"
    _description = 'Class-Category of hazard'
    
    _columns = {
        'name': fields.char(size=128, string='Class-Category', required=True),
        'type_id': fields.many2one('clp.hazard.type', 'Type of hazard', required=True),
        'class_id': fields.many2one('clp.hazard.class', 'Class of hazard', required=True),
    }
    _order = 'type_id, class_id, id'

clp_hazard_class_category()

class clp_hazard_picto(osv.osv):
    _name = "clp.hazard.picto"
    _description = 'Pictograms of hazard'
    
    _columns = {
        'name': fields.char(size=16, string='Code', translate=True, required=True),
        'description': fields.char(size=64, string='Description', translate=True, required=True),
        'image_jpeg_150x150':fields.binary('Pictogram JPEG 150x150'),
        'image_png_236x236':fields.binary('Pictogram PNG 236x236'),
    }
    _order = 'name'

clp_hazard_picto()

class clp_hazard_statement(osv.osv):
    _name = "clp.hazard.statement"
    _description = 'Hazard statements'
    
    _columns = {
        'name': fields.char(size=64, string='Code', required=True),
        'description': fields.char(size=256, string='Description', translate=True, required=True),
        'type_id': fields.many2one('clp.hazard.type', 'Type of hazard', required=True),
    }
    _order = 'type_id, name'

clp_hazard_statement()

class clp_additional_hazard_statement(osv.osv):
    _name = "clp.additional.hazard.statement"
    _description = 'Additional hazard statements'
    
    _columns = {
        'name': fields.char(size=64, string='Code', required=True),
        'description': fields.char(size=256, string='Description', translate=True, required=True),
        'type_id': fields.many2one('clp.hazard.type', 'Type of hazard'),
    }
    _order = 'type_id, name'

clp_additional_hazard_statement()

class clp_precautionary_type(osv.osv):
    _name = "clp.precautionary.type"
    _description = 'Type of precautionary'
    
    _columns = {
        'name': fields.char(size=64, string='Name', translate=True, required=True),
    }
    _order = 'id'

clp_precautionary_type()

class clp_precautionary_statement(osv.osv):
    _name = "clp.precautionary.statement"
    _description = 'Precautionary statements'
    
    _columns = {
        'name': fields.char(size=64, string='Code', required=True),
        'description': fields.char(size=256, string='Description', translate=True, required=True),
        'conditions_for_use': fields.char(size=256, string='Conditions for use', translate=True),
        'type_id': fields.many2one('clp.precautionary.type', 'Type of precautionary', required=True),
    }
    _order = 'type_id, name'

clp_precautionary_statement()

class clp_line(osv.osv):
    _name = "clp.line"
    _description = 'CLP line'
    
    def _picto_list(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_picto_list(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = ', '.join(str(x.name) for x in line.picto_ids if x)
            except:
                res[line.id] = ''
        #print "/***_picto_list(clp_line) - res ", res
        return res
    
    def _hazard_stat_description(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_hazard_stat_description(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = line.hazard_stat_id.description
            except:
                res[line.id] = ''
        #print "/***_hazard_stat_description(clp_line) - res ", res
        return res
    
    def _general_list(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_general_list(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = ', '.join(str(x.name) for x in line.precau_stat_general_ids if x)
            except:
                res[line.id] = ''
        #print "/***_general_list(clp_line) - res ", res
        return res
    
    def _prevention_list(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_prevention_list(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = ', '.join(str(x.name) for x in line.precau_stat_prevention_ids if x)
            except:
                res[line.id] = ''
        #print "/***_prevention_list(clp_line) - res ", res
        return res
    
    def _response_list(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_response_list(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = ', '.join(str(x.name) for x in line.precau_stat_response_ids if x)
            except:
                res[line.id] = ''
        #print "/***_response_list(clp_line) - res ", res
        return res
    
    def _storage_list(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_storage_list(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = ', '.join(str(x.name) for x in line.precau_stat_storage_ids if x)
            except:
                res[line.id] = ''
       #print "/***_storage_list(clp_line) - res ", res
        return res
    
    def _disposal_list(self, cr, uid, ids, field_name, arg, context=None):
        #print " ***_disposal_list(clp_line) - ids ", ids, ' - field_name ', field_name, ' - arg ', arg, ' - context ', context
        if context is None: context = {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            try:
                res[line.id] = ', '.join(str(x.name) for x in line.precau_stat_disposal_ids if x)
            except:
                res[line.id] = ''
        #print "/***_disposal_list(clp_line) - res ", res
        return res
    
    _columns = {
        'name': fields.char(size=64, string='Name', required=True),
        'category': fields.char(size=64, string='Category', translate=True, required=True),
        'class_category_hazard_id': fields.many2one('clp.hazard.class.category', 'Class-Category of hazard'),
        'type_hazard_id': fields.related('class_category_hazard_id', 'type_id', type='many2one', relation='clp.hazard.type', string='Type of hazard', store=True),
        'class_hazard_id': fields.related('class_category_hazard_id', 'class_id', type='many2one', relation='clp.hazard.class', string='Class of hazard', store=True),
        'signal_word': fields.selection([('danger','Danger'),('warning','Warning')], string='Signal word'),
        'picto_ids': fields.many2many('clp.hazard.picto', 'clp_line_picto_rel', 'clp_line_id','picto_id','Pictograms'),
        'picto_list': fields.function(_picto_list, method=True, type='char', size=64, string="Pictograms"),
        
        
        'hazard_stat_id': fields.many2one('clp.hazard.statement','Hazard statements'),
        'description_hazard_stat': fields.function(_hazard_stat_description, method=True, type='char', size=256, string="Description"),
        'precau_stat_general_ids': fields.many2many('clp.precautionary.statement', 'clp_line_precau_general_rel', 'clp_line_id','precau_general_id',
                                                       'Precautionary prevention statements'),
        'general_list': fields.function(_general_list, method=True, type='char', size=128, string="Precautionary general statements"),
        'precau_stat_prevention_ids': fields.many2many('clp.precautionary.statement', 'clp_line_precau_prevention_rel', 'clp_line_id','precau_prevention_id',
                                                       'Precautionary prevention statements'),
        'prevention_list': fields.function(_prevention_list, method=True, type='char', size=128, string="Precautionary prevention statements"),
        'precau_stat_response_ids': fields.many2many('clp.precautionary.statement', 'clp_line_precau_response_rel', 'clp_line_id','precau_response_id',
                                                       'Precautionary response statements'),
        'response_list': fields.function(_response_list, method=True, type='char', size=128, string="Precautionary response statements"),
        'precau_stat_storage_ids': fields.many2many('clp.precautionary.statement', 'clp_line_precau_storage_rel', 'clp_line_id','precau_storage_id',
                                                       'Precautionary storage statements'),
        'storage_list': fields.function(_storage_list, method=True, type='char', size=128, string="Precautionary storage statements"),
        'precau_stat_disposal_ids': fields.many2many('clp.precautionary.statement', 'clp_line_precau_disposal_rel', 'clp_line_id','precau_disposal_id',
                                                       'Precautionary disposal statements'),
        'disposal_list': fields.function(_disposal_list, method=True, type='char', size=128, string="Precautionary disposal statements"),
    }
    _order = 'type_hazard_id, class_hazard_id, id'

clp_line()


