# -*- coding: utf-8 -*-
##############################################################################
#
#    Module Chemicals CLP for Chemical Industries (OpenERP)
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

from osv import fields, osv
import pooler
from tools.translate import _
import decimal_precision as dp



class product_product(osv.osv):
    _inherit = 'product.product'
    _name = 'product.product'

    _columns = {
        'hd_chemical_name': fields.char('Chemical name', size=128, translate=True, select=True),
        'hd_cpl_type': fields.selection([('substance', 'Substance'), ('mixture', 'Mixture')], 'Product Type', required=True),
        'hd_density': fields.float('Density', digits_compute=dp.get_precision('Product UoM 10 moins 5')),
        'hd_molar_mass': fields.float('Molar mass (g/mol)', digits_compute=dp.get_precision('Product UoM 10 moins 5')),
        'hd_concentration': fields.float('Concentration (%)'),
        'hd_ec_number': fields.char('EC No (EINECS/ELINCS/NLP)', size=9, help="The EC number, i.e. EINECS, ELINCS or NLP, is the official number of the substance within the European Union.\n \
                                                                            The EINECS number can be obtained from the European Inventory of Existing Commercial Chemical Substance.\n \
                                                                            The ELINCS number can be obtained from the European List of Notified Substances.\n \
                                                                            The NLP number can be obtained from the list of ‘No-longer-polymers’.\n \
                                                                            The EC number is a seven-digit system of the type XXX-XXX-X which starts at 200-001-8 (EINECS), at 400-010-9 (ELINCS) and at 500-001-0 (NLP)"),
        'hd_cas_number': fields.char('CAS No', size=12, help="Chemical Abstracts Service."),
        'hd_index_number': fields.char('Index No', size=12, help="The Index number for each substance is in the form of a digit sequence of the type ABC-RST-VW-Y.\n \
                                                                            ABC corresponds to the atomic number of the most characteristic element or the most characteristic organic group in the molecule.\n \
                                                                            RST is the consecutive number of the substance in the series ABC.\n \
                                                                            VW denotes the form in which the substance is produced or placed on the market.\n \
                                                                            Y is the check-digit calculated in accordance with the 10-digit ISBN method."),
        'hd_clp_line_ids': fields.many2many('clp.line', 'product_clp_line_rel', 'product_id','clp_line_id',
                                                       'CLP Lines'),
        'hd_product_hazard_lines' : fields.one2many('product.clp.label.hazard.line', 'product_id', 'Hazard statements'),
        'hd_product_picto_lines' : fields.one2many('product.clp.label.picto.line', 'product_id', 'Pictograms'),
        'hd_signal_word': fields.selection([('danger','Danger'),('warning','Warning')], string='Signal word'),
        'hd_product_precau_lines' : fields.one2many('product.clp.label.precau.line', 'product_id', 'Precautionary statements'),
        'hd_product_additional_lines' : fields.one2many('product.clp.label.additional.line', 'product_id', 'Additional Hazard Statements'),
    }
    _defaults = {
        'hd_cpl_type': 'mixture',
        'hd_density': lambda *a: 1.0,
    }

    def button_init_label_infos(self, cr, uid, ids, context=None, properties=[]):
        #print " ***button_init_label_infos(product_product) - ids ", ids, " - context ", context
        if context is None: context = {}
        list_picto = []
        list_signal_word = []
        list_hazard = []
        list_precau = []
        picto_line_obj = self.pool.get('product.clp.label.picto.line')
        hazard_line_obj = self.pool.get('product.clp.label.hazard.line')
        additional_line_obj = self.pool.get('product.clp.label.additional.line')
        precau_line_obj = self.pool.get('product.clp.label.precau.line')
        product = self.browse(cr, uid, ids[0])
        for clp_line in product.hd_clp_line_ids:
            if clp_line.signal_word: list_signal_word.append(clp_line.signal_word)
            list_picto.extend([x.id for x in clp_line.picto_ids])
            list_hazard.append(clp_line.id)
            list_precau.extend([x.id for x in clp_line.precau_stat_general_ids])
            list_precau.extend([x.id for x in clp_line.precau_stat_prevention_ids])
            list_precau.extend([x.id for x in clp_line.precau_stat_response_ids])
            list_precau.extend([x.id for x in clp_line.precau_stat_storage_ids])
            list_precau.extend([x.id for x in clp_line.precau_stat_disposal_ids])
            
        list_signal_word = list(set(list_signal_word))#pour supprimer les doublons
        list_picto = list(set(list_picto))#pour supprimer les doublons
        list_hazard = list(set(list_hazard))#pour supprimer les doublons
        list_precau = list(set(list_precau))#pour supprimer les doublons
        # traitement signal word
        signal_word = False
        if len(list_signal_word) == 2: signal_word = 'danger'
        elif len(list_signal_word) == 1: signal_word = list_signal_word[0]
        self.write(cr, uid, ids, {'hd_signal_word': signal_word})
        #supprimer les lignes de picto existantes
        picto_line_obj.unlink(cr, uid, [x.id for x in product.hd_product_picto_lines])
        #creer les ligne de picto
        for picto_id in list_picto:
            picto_line_obj.create(cr, uid, {'product_id': ids[0], 'picto_id': picto_id})
        #supprimer les lignes de hazard existantes
        hazard_line_obj.unlink(cr, uid, [x.id for x in product.hd_product_hazard_lines])
        #creer les ligne de hazard
        for hazard_id in list_hazard:
            hazard_line_obj.create(cr, uid, {'product_id': ids[0], 'hazard_id': hazard_id})
        #supprimer les lignes additionnelles existantes
        additional_line_obj.unlink(cr, uid, [x.id for x in product.hd_product_additional_lines])
        #supprimer les lignes de precau existantes
        precau_line_obj.unlink(cr, uid, [x.id for x in product.hd_product_precau_lines])
        #creer les ligne de precau
        for precau_id in list_precau:
            precau_line_obj.create(cr, uid, {'product_id': ids[0], 'precau_id': precau_id})
        #print "/***button_init_label_infos(product_product)"
        return True

product_product()

class product_clp_label_picto_line(osv.osv):
    _description='Product Pictogram Line'
    _name = 'product.clp.label.picto.line'
    _columns = {
        'product_id': fields.many2one('product.product', 'Product'),
        'sequence': fields.integer('Seq'),
        'picto_id': fields.many2one('clp.hazard.picto', 'Code'),
        'name': fields.related('picto_id', 'name', type='char', size=64, string='Code', readonly=True),
        'description': fields.related('picto_id', 'description', type='char', size=64, string='Description', readonly=True),
        'image_jpeg_150x150': fields.related('picto_id', 'image_jpeg_150x150', type='binary', string='Pictogram JPEG 150x150', readonly=True),
        'image_png_236x236': fields.related('picto_id', 'image_png_236x236', type='binary', string='Pictogram PNG 236x236', readonly=True),
    }
    _order = 'sequence, id'
    _sql_constraints = [
        ('picto_id_uniq', 'unique (picto_id,product_id)', 'This pictogram is already used !')
        ]

product_clp_label_picto_line()

class product_clp_label_hazard_line(osv.osv):
    _description='Product Hazard statements Line'
    _name = 'product.clp.label.hazard.line'
    _columns = {
        'product_id': fields.many2one('product.product', 'Product'),
        'sequence': fields.integer('Seq'),
        'hazard_id': fields.many2one('clp.line', 'Class-Category'),
        'hazard_stat_id': fields.related('hazard_id', 'hazard_stat_id', type='many2one', relation='clp.hazard.statement', string='Code', readonly=True),
        'description': fields.related('hazard_id', 'description_hazard_stat', type='char', size=256, string='Description', readonly=True),
        'type_hazard_id': fields.related('hazard_id', 'type_hazard_id', type='many2one', relation='clp.hazard.type', string='Type', readonly=True),
        'class_hazard_id': fields.related('hazard_id', 'class_hazard_id', type='many2one', relation='clp.hazard.class', string='Class', readonly=True),
    }
    _order = 'sequence, id'
    _sql_constraints = [
        ('hazard_id_uniq', 'unique (hazard_id,product_id)', 'This Hazard statement is already used !')
        ]

product_clp_label_hazard_line()


class product_clp_label_additional_line(osv.osv):
    _description='Product Additional Hazard Statements Line'
    _name = 'product.clp.label.additional.line'
    _columns = {
        'product_id': fields.many2one('product.product', 'Product'),
        'sequence': fields.integer('Seq'),
        'additional_id': fields.many2one('clp.additional.hazard.statement', 'Code'),
        'description': fields.related('additional_id', 'description', type='char', size=256, string='Description', readonly=True),
    }
    _order = 'sequence, id'
    _sql_constraints = [
        ('additonal_id_uniq', 'unique (additional_id,product_id)', 'This Additional Hazard Statement is already used !')
        ]

product_clp_label_additional_line()

class product_clp_label_precau_line(osv.osv):
    _description='Product Precautionary statements Line'
    _name = 'product.clp.label.precau.line'
    _columns = {
        'product_id': fields.many2one('product.product', 'Product'),
        'sequence': fields.integer('Seq'),
        'precau_id': fields.many2one('clp.precautionary.statement', 'Code'),
        'description': fields.related('precau_id', 'description', type='char', size=256, string='Description', readonly=True),
        'type_precau_id': fields.related('precau_id', 'type_id', type='many2one', relation='clp.precautionary.type', string='Type', readonly=True),
    }
    _order = 'sequence, id'
    _sql_constraints = [
        ('precau_id_uniq', 'unique (precau_id,product_id)', 'This Precautionary statement is already used !')
        ]

product_clp_label_precau_line()

