# -*- coding: utf-8 -*-
##############################################################################
#
#    Module Laboratory for Process Industries (OpenERP)
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

import datetime
import time
from osv import osv, fields
import pooler
import netsvc
import math

#hdcm 20130704 import import ir
from tools import config
from tools.translate import _


# Definition de la classe expression des resultats pour un test
class labo_test_express_res(osv.osv):
    _description='Test expression of results'
    _name = 'labo.test.express.res'
    _columns = {
        'name' : fields.char('Expression of result', size=64, required=True),
        'active': fields.boolean('Active', help="The active field allows you to hide the Expression of results without removing it."),
    }
    _order = 'name'
    _sql_constraints = [
        ('name_uniq', 'unique (name)', 'The name of the Expression of result must be unique !')
    ]
    _defaults = {
        'active': lambda *a: 1,
    }

labo_test_express_res()

# Definition de la classe categorie pour un test
class labo_test_category(osv.osv):
    _description='Test Categories'
    _name = 'labo.test.category'
    _columns = {
        'name' : fields.char('Test category', size=64, required=True),
        'active': fields.boolean('Active', help="The active field allows you to hide the Test category test without removing it."),
        'sequence': fields.integer('Sequence'),
    }
    _order = 'sequence, id'
    _sql_constraints = [
        ('name_uniq', 'unique (name)', 'The name of the Test category must be unique !')
    ]
    _defaults = {
        'active': lambda *a: 1,
    }

labo_test_category()

# Definition de la classe methode pour un test
class labo_test_method(osv.osv):
    _description='Test methods'
    _name = 'labo.test.method'
    _columns = {
        'name' : fields.char('Test method', size=64, required=True),
        'active': fields.boolean('Active', help="The active field allows you to hide the Test method without removing it."),
        'note' : fields.text('Note'),
    }
    _order = 'name'
    _sql_constraints = [
        ('name_uniq', 'unique (name)', 'The name of the Test method must be unique !')
    ]
    _defaults = {
        'active': lambda *a: 1,
    }

labo_test_method()


# Definition de la classe test
class labo_test(osv.osv):
    _description='Tests'
    _name = 'labo.test'
    _columns = {
        'name' : fields.char('Test', size=256, required=True, translate=True),
        'code' : fields.char('Code', size=64),
        'expres_id': fields.many2one('labo.test.express.res', 'Expression of results', required=True),
        'test_category_id': fields.many2one('labo.test.category', 'Test category'),
        'note' : fields.text('Note'),
        'active': fields.boolean('Active', help="The active field allows you to hide the test without removing it."),
    }
    _order = 'name'
    _defaults = {
        'active': lambda *a: 1,
    }

labo_test()

# Definition de la classe modéle d'analyse pour un produit
class labo_analysis_template(osv.osv):
    _description='Template Analysis'
    _name = 'labo.analysis.template'
    _columns = {
        'name' : fields.char('Name', size=128, required=True, readonly=True, states={'draft': [('readonly', False)]}),
        #hdcm 2013_09_13 'revision': fields.integer('Revision', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'revision': fields.integer('Revision', readonly=True),
        'type_template_analysis': fields.selection((('purchase','Purchase'),('customer_return','Customer return'),('sale','Sale'),('production','Production'),('internal','Internal'),),'Type template analysis', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'product_id': fields.many2one('product.product', 'Product', domain=[('hd_track_lot', '=', True)], required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'analysis_template_line_ids' : fields.one2many('labo.analysis.template.line', 'labo_analysis_template_id', 'Tests', readonly=True, states={'draft': [('readonly', False)]}),
        'active': fields.boolean('Active', help="The active field allows you to hide the analysis template without removing it.", readonly=True, states={'draft': [('readonly', False)]}),
        'state': fields.selection([('draft','Draft'),('ready_to_approve','Ready to approve'),('approved','Approved'),('unused','Unused')],'Status', readonly=True),
        'date_writing': fields.datetime('Writing date', readonly=True),
        'author_id': fields.many2one('res.users', 'Author', readonly=True),
        'date_approval': fields.datetime('Approval date', readonly=True),
        'approver_id': fields.many2one('res.users', 'Approver', readonly=True),
        'note' : fields.text('Note', readonly=True, states={'draft': [('readonly', False)]}),
    }
    _order = 'name'
    _sql_constraints = [
        ('product_type_name_revision_uniq', 'unique (product_id, type_template_analysis, name, revision)', 'The product/type/name/revision must be unique !')
    ]
    _defaults = {
        'active': lambda *a: 1,
        'state': lambda *a: 'draft',
        'date_writing': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'author_id': lambda x,y,z,c: z,
        'revision': lambda *a: 0,
    }

    def unlink(self, cr, uid, ids, context={}):
        #Allows to delete 'labo_analysis_template' in draft state
        for rec in self.browse(cr, uid, ids, context=context):
            if rec.state not in ['draft']:
                raise osv.except_osv(_('Invalid action !'), _('Cannot delete a Template Analysis which is %s !')%(rec.state,))
        return super(labo_analysis_template, self).unlink(cr, uid, ids, context=context)

    def copy(self, cr, uid, id, default=None,context=None):
        #print "***copy(labo_analysis_template) - default ",default
        raise osv.except_osv(_('Invalid action !'), _('To copy a Template Analysis, used button \'New revision\' !'))

    def button_new_revision(self, cr, uid, ids, context={}):
        analysis_template_org_obj = self.browse(cr, uid, ids[0], context=context)
        default = {}
        default.update({
                        'name': analysis_template_org_obj.name,
                        'revision': analysis_template_org_obj.revision + 1,
                        'author_id' : uid,
                        'date_writing' : time.strftime('%Y-%m-%d %H:%M:%S'),
                        'approver_id' : False,
                        'date_approval' : False
                        })
        return super(labo_analysis_template, self).copy(cr, uid, analysis_template_org_obj.id, default, context)

    '''hdcm 2013_09_13 v604 def product_id_change(self, cr, uid, ids, product_id, state, name= '', revision = 0):
        #print '***product_id_change(labo_template_analysis) - ids ',ids,' - product_id ',product_id," - state ",state,' - name ',name," - revision ",revision
        result = {}
        warning = {}
        if  product_id and ((state == 'draft') or (state == False)) and (revision == 0):
            product_obj = self.pool.get('product.product').browse(cr, uid, product_id)
            result['name'] = 'Analysis ' + product_obj.name
        elif ids:
            analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, ids[0])
            result['product_id'] = analysis_template_obj.product_id.id
            warning={'title':'Information', 'message':'You can not edit this field'}
        #print '/***product_id_change(labo_template_analysis) - value ',result
        return {'value':result, 'warning':warning}'''
    
    def product_id_change(self, cr, uid, ids, product_id, type, state, name= '', revision = 0):
        #print '***product_id_change(labo_template_analysis) - ids ',ids,' - product_id ',product_id," - type ",type," - state ",state,' - name ',name," - revision ",revision
        result = {}
        warning = {}
        if not (product_id and type):
            result['name'] = ' '
            result['type_template_analysis'] = False
            return {'value':result, 'warning':warning}
        if  product_id and ((state == 'draft') or (state == False)) and (revision == 0):
            product_obj = self.pool.get('product.product').browse(cr, uid, product_id)
            result['name'] = '['+ type + '] ' + 'Analysis ' + product_obj.name
        elif ids:
            analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, ids[0])
            result['product_id'] = analysis_template_obj.product_id.id
            warning={'title':'Information', 'message':'You can not edit this field'}
        #print '/***product_id_change(labo_template_analysis) - value ',result
        return {'value':result, 'warning':warning}

    def button_ready_to_approve(self, cr, uid, ids):
        self.write(cr, uid, ids, {'author_id': uid,'date_writing': time.strftime('%Y-%m-%d %H:%M:%S'),'state': 'ready_to_approve'})
        return True

    def button_approved(self, cr, uid, ids):
        # passer la revision precedente dans l'etat 'unused'
        analysis_template_org_obj = self.browse(cr, uid, ids[0])
        for analysis_template_approved_id in self.search(cr, uid, [('product_id', '=', analysis_template_org_obj.product_id.id),
                                                                   ('type_template_analysis', '=', analysis_template_org_obj.type_template_analysis), ('name', '=',analysis_template_org_obj.name)]):
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'labo.analysis.template', analysis_template_approved_id, 'button_unused', cr)
        self.write(cr, uid, ids, {'approver_id': uid,'date_approval': time.strftime('%Y-%m-%d %H:%M:%S'),'state': 'approved'})
        return True

    def button_draft(self, cr, uid, ids):
        if ids:
            # verifier que labo_analysis_template n'est pas deja utilise
            if self.pool.get('labo.analysis').search(cr, uid, [('analysis_template_id', '=', ids[0])]):
                raise osv.except_osv(_('Invalid action !'), _('This Template Analysis is allready used !'))
            else:
                self.write(cr, uid, ids, {'author_id': uid,'date_writing': time.strftime('%Y-%m-%d %H:%M:%S'),'approver_id': False,'date_approval': False,
                                         'state': 'draft'})
                return True
        else:
            return False

    def button_unused(self, cr, uid, ids):
        self.write(cr, uid, ids, {'state': 'unused'})
        return True

labo_analysis_template()

# Definition de la classe lignes de modele d'analyse pour un produit
class labo_analysis_template_line(osv.osv):
    _description='Template Analysis Line'
    _name = 'labo.analysis.template.line'
    _columns = {
        'labo_analysis_template_id': fields.many2one('labo.analysis.template', 'Template analysis Ref', required=True, ondelete='cascade'),
        'sequence': fields.integer('Sequence'),
        'test_id' : fields.many2one('labo.test', 'Test'),
        'name': fields.char('Test', size=256, required=True),
        'specification_range': fields.text('Specification range', required=True),
        'expres_id': fields.many2one('labo.test.express.res', 'Expression of results'),
        'test_category_id': fields.many2one('labo.test.category', 'Test category'),
        'labo_test_method_ids': fields.many2many('labo.test.method', 'labo_analysis_template_method_rel', 'labo_analysis_template_line_id','labo_test_method_id','Test methods'),
        'note' : fields.text('Note'),
    }
    _order = 'sequence, id'

    _defaults = {
        'sequence': lambda *a: 10,
    }

    def test_id_change(self, cr, uid, ids, test_id, product_id=False, name= '', revision= -1):
        #print '***test_id_change(labo_analysis_template_line) - ids ',ids,' - test_id ',test_id,' - product_id ',product_id,' - name ',name,\
        #        ' - revision ',revision
        if not(product_id and name and (revision >= 0)):
            raise osv.except_osv(_('Invalid action !'), _('You have to select a product/name/revision in the Template Analysis form !'))
        result = {}
        if  test_id:
            test_obj = self.pool.get('labo.test').browse(cr, uid, test_id)
            result['name'] = test_obj.name
            result['expres_id'] = test_obj.expres_id.id
            result['test_category_id'] = test_obj.test_category_id.id
            result['note'] = test_obj.note
            result['specification_range'] = False
        #print '/***test_id_change(labo_analysis_template_line) - value ',result
        return {'value':result}

labo_analysis_template_line()

# Definition de la classe analyse pour un lot
class labo_analysis(osv.osv):
    _description='Analysis'
    _name = 'labo.analysis'
    _columns = {
        'name' : fields.char('Description', size=128, required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'code' : fields.char('Code', size=16, readonly=True),
        'code_org' : fields.char('Code org', size=16, readonly=True),
        'product_id': fields.many2one('product.product', 'Product', domain=[('hd_track_lot', '=', True)], readonly=True, states={'draft': [('readonly', False)]}),
        #hdcm 2013_09_13 v70 'product_id': fields.related('analysis_template_id', 'product_id', type='many2one', relation='product.product', string='Product', store=True, readonly=True),
        'lot_id': fields.many2one('stock.production.lot', 'Lot', domain=['|',('hd_qty_real','<>',0.0),('hd_qty_virt','<>',0.0),'!',('state', '=', 'destroyed'),('hd_qty_real','<>',0.0)],
                                  readonly=True, states={'draft': [('readonly', False)]}),
        'production_id': fields.many2one('mrp.production', 'Production', domain=[('state', '=', 'in_production')],
                                  readonly=True, states={'draft': [('readonly', False)]}),
        'analysis_template_id': fields.many2one('labo.analysis.template', 'Template Analysis', domain=[('state', '=', 'approved')], required=True, readonly=True, states={'draft': [('readonly', False)]}),
        #hdcm 2013_09_13 'revision': fields.integer('Revision', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'revision': fields.related('analysis_template_id', 'revision', type='integer', string='Revision', store=True, readonly=True),#hdcm 2013_02_05
        'type_analysis': fields.selection((('purchase','Purchase'),('customer_return','Customer return'),('sale','Sale'),('production','Production'),('internal','Internal'),),'Type', readonly=True),
        #'analysis_line_ids' : fields.one2many('labo.analysis.line', 'labo_analysis_id', 'Tests', readonly=True),
        'analysis_line_ids' : fields.one2many('labo.analysis.line', 'labo_analysis_id', 'Tests'),
        'conform': fields.boolean('Conform', readonly=True),
        'date_create': fields.datetime('Create date', readonly=True),
        'confirm_by_id': fields.many2one('res.users', 'Confirm by', readonly=True),
        'date_start': fields.datetime('Start date', readonly=True),
        'approver_id': fields.many2one('res.users', 'Approver', readonly=True),
        'date_end': fields.datetime('End date', readonly=True),
        'note' : fields.text('Note', readonly=True, states={'draft': [('readonly', False)],'in_progress': [('readonly', False)]}),
        'state': fields.selection([('draft','Draft'),('in_progress','In progress'),('done','Done'),('cancel','Cancelled')],'Status', readonly=True),
    }
    _order = 'code'
    _sql_constraints = [
        ('code_uniq', 'unique (code)', 'The code must be unique !')
    ]
    _defaults = {
        'date_create': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'state': lambda *a: 'draft',
        'conform': lambda *a: 0,
    }

    def unlink(self, cr, uid, ids, context={}):
        #interdire la suppression d'une analyse pour conserver la continuite des codes
        #une analyse peut etre annulee
        for rec in self.browse(cr, uid, ids, context=context):
            raise osv.except_osv(_('Invalid action !'), _('Cannot delete an Analysis !'))

    def create(self, cr, user, vals, context=None):
        #print "***create(labo_analysis) - vals ",vals," - context ",context
        analysis_id = super(labo_analysis, self).create(cr, user, vals, context)
        self.write(cr,user, [analysis_id], {'code':'%07d'%(analysis_id)})
        if ('production_id' in vals) and vals['production_id']:
            self.write(cr,user, [analysis_id], {'product_id':self.pool.get('mrp.production').browse(cr, user, vals['production_id']).product_id.id})
        if ('analysis_template_id' in vals) and vals['analysis_template_id']:
            self.write(cr,user, [analysis_id], {'type_analysis':self.pool.get('labo.analysis.template').browse(cr, user, vals['analysis_template_id']).type_template_analysis})
        # creer les lignes d'analyse
        self.fill_labo_analysis_lines(cr, user, [analysis_id], self.browse(cr, user, analysis_id).analysis_template_id.id)
        #print "/***create(labo_analysis)"
        return analysis_id

    def write(self, cr, uid, ids, vals, context=None):
        #print "***write(labo_analysis) - ids ",ids," - vals ",vals," - context ",context

        #print "/***write(labo_analysis)"
        return super(labo_analysis, self).write(cr, uid, ids, vals, context=context)

    def copy(self, cr, uid, id, default=None,context=None):
        #print "***copy(labo_analysis) - id ",id," - context ",context
        if not default:
            default = {}
        default.update({'code': False, 'analysis_line_ids': []})
        #print "default ",default
        #print "/***copy(labo_analysis)"
        return super(labo_analysis, self).copy(cr, uid, id, default, context)


    def button_confirm(self, cr, uid, ids):
        if ids:
            self.write(cr, uid, ids, {'confirm_by_id': uid,'date_start': time.strftime('%Y-%m-%d %H:%M:%S'),
                                      'state': 'in_progress'})
            return True
        else:
            return False

    def button_done(self, cr, uid, ids):
        if ids:
            # uniquement si tous les tests de cette analyse sont dans l'etat 'valid_controler'
            for analysis_line_obj in self.browse(cr, uid, ids[0]).analysis_line_ids:
                if not (analysis_line_obj.state == 'valid_controler'):
                    raise osv.except_osv(_('Invalid action !'), _('All lines must be in the state \'Valid controler\' !'))
            # passage en 'conform' si toutes les lignes sont conformes
            conform = True
            for analysis_line_obj in self.browse(cr, uid, ids[0]).analysis_line_ids:
                #print "analysis_line_obj.conform ",analysis_line_obj.conform
                if not analysis_line_obj.conform:
                    conform = False
                    break
            self.write(cr, uid, ids, {'approver_id': uid,'date_end': time.strftime('%Y-%m-%d %H:%M:%S'),
                                      'conform': conform, 'state': 'done'})
            # passage des lignes dans l'etat 'done'
            self.pool.get('labo.analysis.line').write(cr, uid, [x.id for x in self.browse(cr, uid, ids[0]).analysis_line_ids], {'state': 'done'})
            wf_service = netsvc.LocalService("workflow")
            for analysis_line_obj in self.browse(cr, uid, ids[0]).analysis_line_ids:
                wf_service.trg_validate(uid, 'labo.analysis.line', analysis_line_obj.id, 'button_done', cr)
            return True
        else:
            return False

    def button_draft(self, cr, uid, ids):
        if ids:
            # uniquement si tous les tests de cette analyse sont dans l'etat 'draft'
            for analysis_line_obj in self.browse(cr, uid, ids[0]).analysis_line_ids:
                if not (analysis_line_obj.state == 'draft'):
                    raise osv.except_osv(_('Invalid action !'), _('All lines must be in the state \'Draft\' !'))
            self.write(cr, uid, ids, {'confirm_by_id': False,'date_start': False,
                                      'state': 'draft'})
            return True
        else:
            return False

    def button_cancel(self, cr, uid, ids):
        if ids:
            # uniquement si l'analyse n'est pas dans l'etat 'done'
            if self.browse(cr, uid, ids[0]).state == 'done':
                raise osv.except_osv(_('Invalid action !'), _('This Analysis is in state \'Done\' !'))
            else:
                self.write(cr, uid, ids, {'approver_id': uid,'date_end': time.strftime('%Y-%m-%d %H:%M:%S'),
                                      'state': 'cancel'})
                # passage des lignes dans l'etat 'cancel'
                self.pool.get('labo.analysis.line').write(cr, uid, [x.id for x in self.browse(cr, uid, ids[0]).analysis_line_ids], {'state': 'cancel'})
                wf_service = netsvc.LocalService("workflow")
                for analysis_line_obj in self.browse(cr, uid, ids[0]).analysis_line_ids:
                    wf_service.trg_validate(uid, 'labo.analysis.line', analysis_line_obj.id, 'button_cancel', cr)
                return True
        else:
            return False

    def button_fill_tests(self, cr, uid, ids, context={}):
        #print "***button_fill_tests(labo_analysis) - ids ",ids," - context ",context
        if ids:
            self.write(cr, uid, ids[0], {})
            #print "/***button_fill_tests(labo_analysis)"
        return True


    #hdcm 2013_09_20 v604 def product_id_change(self, cr, uid, ids, product_id):
    def product_id_change(self, cr, uid, ids, product_id, context=None):
        #hdcm 2013_09_20 v604 print "***product_id_change(labo_analysis)  ", ids," - product_id ",product_id
        #print "***product_id_change(labo_analysis)  ", ids," - product_id ",product_id," - context ",context
        if context is None: context = {}
        domain = {}
        result = {}
        warning = {}
        if not product_id:
            result['lot_id'] = False
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            # suppression des lignes
            self.delete_labo_analysis_lines(cr, uid, ids)
            return {'value':result, 'warning':warning}
        # etablir le domaine pour les lots
        '''hdcm 2013_09_16 v604 domain['lot_id'] = [('product_id','=', product_id),'|',('hd_qty_real','<>',0.0),('hd_qty_virt','<>',0.0),
                            '!',('state', '=', 'destroyed'),('hd_qty_real','<>',0.0)]'''
        domain['lot_id'] = [('product_id','=', product_id),'|',('hd_qty_real','<>',0.0),('hd_qty_virt','<>',0.0),
                            '!',('state', '=', 'destroyed'),('hd_qty_real','<>',0.0), ('hd_type', 'not in', ('int', 'inv'))]
        # etablir le domaine pour les analysis template
        domain['analysis_template_id'] = [('product_id','=', product_id),('state', '=', 'approved')]
        #hdcm 2013_09_20 v604 result['lot_id'] = False
        if not context.get('default_lot_id', False):
            result['lot_id'] = False
        result['analysis_template_id'] = False
        #hdcm 2013_09_13 v604 result['revision'] = ''
        result['name'] = ''
        # suppression des lignes
        self.delete_labo_analysis_lines(cr, uid, ids)
        #print "/***product_id_change(labo_analysis) - value ",result," - domain ",domain
        return {'value':result, 'domain':domain, 'warning':warning}

    def lot_id_change(self, cr, uid, ids, product_id, lot_id, production_id):
        #print "***lot_id_change(labo_analysis)  ", ids," - product_id ",product_id
        domain = {}
        result = {}
        warning = {}
        if production_id:
            result['lot_id'] = False
            warning={'title':'Invalid action', 'message':'This analysis is defined in production'}
            return {'value':result, 'warning':warning}
        if lot_id and not product_id:
            # recuperer le product_id de ce lot
            product_obj_id = self.pool.get('stock.production.lot').browse(cr, uid, lot_id).product_id.id
            # etablir le domaine pour les analysis template
            domain['analysis_template_id'] = [('product_id','=', product_obj_id),('state', '=', 'approved')]
            result['product_id'] = product_obj_id
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            # suppression des lignes
            self.delete_labo_analysis_lines(cr, uid, ids)
            return {'value':result, 'domain':domain}
        if lot_id and self.pool.get('stock.production.lot').production_running(cr, uid,[lot_id]):
            result['lot_id'] = False
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            # suppression des lignes
            self.delete_labo_analysis_lines(cr, uid, ids)
            warning={'title':'Invalid action', 'message':'The MO is \'In Production\''}
            return {'value':result, 'warning':warning}

        if not lot_id:
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            # suppression des lignes
            self.delete_labo_analysis_lines(cr, uid, ids)
            return {'value':result, 'warning':warning}
        #print "/***lot_id_change(labo_analysis) - value ",result," - domain ",domain
        return {'value':result, 'domain':domain, 'warning':warning}

    def production_id_change(self, cr, uid, ids, production_id, lot_id, context={}):
        #print "***production_id_change(labo_analysis) -ids ", ids," - production_id ", production_id
        result = {}
        warning = {}
        if not ids:
            result['production_id'] = False
        elif ids and lot_id:
            result['production_id'] = False
        else:
            result['production_id'] = production_id
        warning={'title':'Information', 'message':'You can not edit this field'}
        #print "/***production_id_change(labo_analysis) - result ",result
        return {'value':result, 'warning':warning}

    def analysis_template_id_change(self, cr, uid, ids, product_id, lot_id, production_id, analysis_template_id, context={}):
        #print "***analysis_template_id_change(labo_analysis) -ids ", ids," - product_id ",product_id," - lot_id ",lot_id
        domain = {}
        result = {}
        warning = {}
        if ids and production_id and not lot_id:
            result['analysis_template_id'] = self.browse(cr, uid, ids[0]).analysis_template_id.id
            warning={'title':'Invalid action', 'message':'This analysis is defined in production'}
            return {'value':result, 'warning':warning}
        if not product_id:
            result['lot_id'] = False
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            # suppression des lignes
            if ids:
                self.delete_labo_analysis_lines(cr, uid, ids)
            warning={'title':'Information', 'message':'You must edit first the product field'}
            return {'value':result, 'warning':warning}
        if not lot_id:
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            # suppression des lignes
            if ids:
                self.delete_labo_analysis_lines(cr, uid, ids)
            warning={'title':'Information', 'message':'You must edit the lot field'}
            return {'value':result, 'warning':warning}
        if not analysis_template_id:
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            if ids:
                # suppression des lignes
                self.delete_labo_analysis_lines(cr, uid, ids)
            return {'value':result, 'warning':warning}
        else:
            analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, analysis_template_id)
            product_obj = self.pool.get('product.product').browse(cr, uid, product_id)
            # ecrire la revision
            #hdcm 2013_09_13 v604 result['revision'] = analysis_template_obj.revision
            result['name'] = '['+ analysis_template_obj.type_template_analysis + ']' + ' Analysis ' + product_obj.name
            if ids:
                # suppression des lignes
                self.delete_labo_analysis_lines(cr, uid, ids)
                # creer les lignes d'analyse
                self.fill_labo_analysis_lines(cr, uid, ids, analysis_template_id)

        #print "/***analysis_template_id_change(labo_analysis) - value ",result," - domain ",domain
        return {'value':result, 'domain':domain, 'warning':warning}

    '''hdcm 2013_09_13 v604 def revision_change(self, cr, uid, ids, revision, analysis_template_id, context={}):
        #print "***revision_change(labo_analysis) -ids ", ids," - analysis_template_id ", analysis_template_id," - revision ",revision
        result = {}
        warning = {}
        if analysis_template_id:
            analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, analysis_template_id)
            result['revision'] = analysis_template_obj.revision
        warning={'title':'Information', 'message':'You can not edit this field'}
        #print "/***revision_change(labo_analysis) - result ",result
        return {'value':result, 'warning':warning}'''



    def analysis_template_id_change_mo(self, cr, uid, ids ,production_id, analysis_template_id, context={}):
        #print "***analysis_template_id_change_mo(labo_analysis) -ids ", ids," - production_id ",production_id," - analysis_template_id ",analysis_template_id
        domain = {}
        result = {}
        warning = {}

        product_id2 = self.pool.get('mrp.production').browse(cr, uid, production_id).product_id.id
        # etablir le domaine pour les analysis template
        domain['analysis_template_id'] = [('product_id','=', product_id2),('state', '=', 'approved')]

        if not analysis_template_id:
            result['analysis_template_id'] = False
            #hdcm 2013_09_13 v604 result['revision'] = ''
            result['name'] = ''
            if ids:
                # suppression des lignes
                self.delete_labo_analysis_lines(cr, uid, ids)
            #print "/***analysis_template_id_change_mo(labo_analysis) - value ",result," - domain ",domain
            return {'value':result, 'domain':domain, 'warning':warning}
        else:
            analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, analysis_template_id)
            product_obj = self.pool.get('product.product').browse(cr, uid, product_id2)
            # ecrire la revision
            #hdcm 2013_09_13 v604 result['revision'] = analysis_template_obj.revision
            result['name'] = 'Analysis ' + product_obj.name
            if ids:
                # suppression des lignes
                self.delete_labo_analysis_lines(cr, uid, ids)
                # creer les lignes d'analyse
                self.fill_labo_analysis_lines(cr, uid, ids, analysis_template_id)

            #print "/***analysis_template_id_change_mo(labo_analysis) - value ",result," - domain ",domain
            return {'value':result, 'domain':domain, 'warning':warning}

    '''hdcm 2013_09_13 v604 def revision_change_mo(self, cr, uid, ids, revision, analysis_template_id, context={}):
        #print "***revision_change_mo(labo_analysis) -ids ", ids," - analysis_template_id ", analysis_template_id," - revision ",revision
        result = {}
        warning = {}
        if analysis_template_id:
            analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, analysis_template_id)
            result['revision'] = analysis_template_obj.revision
        if ids:
            warning={'title':'Information', 'message':'You can not edit this field'}
        #print "/***revision_change_mo(labo_analysis) - result ",result
        return {'value':result, 'warning':warning}'''

    def fill_labo_analysis_lines(self,cr, uid, ids, analysis_template_id, context={}):
        #print "***fill_labo_analysis_lines(labo_analysis) - ids ",ids," - analysis_template_id ",analysis_template_id," - context ",context
        if not (ids and analysis_template_id):
            return False
        analysis_obj = self.browse(cr, uid, ids[0])
        analysis_template_obj = self.pool.get('labo.analysis.template').browse(cr, uid, analysis_template_id)
        for analysis_template_line in analysis_template_obj.analysis_template_line_ids:
            labo_analysis_line_id = self.pool.get('labo.analysis.line').create(cr, uid, {'labo_analysis_id' : ids[0],
                        'product_id' : analysis_obj.product_id.id, 'lot_id' : analysis_obj.lot_id.id,
                        'production_id' : analysis_obj.production_id.id,
                        'code_analysis': analysis_obj.code,
                        'sequence' : analysis_template_line.sequence ,'test_id' : analysis_template_line.test_id.id,
                        'name' : analysis_template_line.name,'specification_range' : analysis_template_line.specification_range,
                        'expres_id' : analysis_template_line.expres_id.id,'test_category_id' : analysis_template_line.test_category_id.id
                        })
            for test_method in analysis_template_line.labo_test_method_ids:
                cr.execute('insert into labo_analysis_method_rel (labo_analysis_line_id,labo_test_method_id) values (%s,%s)', (labo_analysis_line_id, test_method.id))
        #print "/***fill_labo_analysis_lines(labo_analysis)"
        return True

    def delete_labo_analysis_lines(self,cr, uid, ids, context={}):
        #print "***delete_labo_analysis_lines(labo_analysis) - ids ",ids," - context ",context
        if not ids:
            return False
        unlink_ids = [x.id for x in self.pool.get('labo.analysis').browse(cr, uid, ids[0]).analysis_line_ids]
        #print "unlink_ids ",unlink_ids
        self.pool.get('labo.analysis.line').unlink(cr, uid, unlink_ids, context=context)
        self.write(cr, uid, ids, {'analysis_line_ids':self.pool.get('labo.analysis').browse(cr, uid, ids[0]).analysis_line_ids})

        #print "/***delete_labo_analysis_lines(labo_analysis)"
        return True

    # copie d'une analyse vers un ou plusieurs lots
    def copy_labo_analysis(self,cr, uid, analysis_org_id, lot_dest_ids, fab=False, context={}):
        #print "***copy_labo_analysis(labo_analysis) - analysis_org_id ",analysis_org_id," - lot_dest_ids ",lot_dest_ids," - context ", context
        analysis_org_obj = self.browse(cr, uid, analysis_org_id)
        for lot_dest_id in lot_dest_ids:
            # creation d'une analyse faisant ref a ce lot
            analysis_new_id = self.create(cr, uid, {
                            'name' : analysis_org_obj.name,
                            'code' : False,
                            'code_org' : analysis_org_obj.code,
                            'product_id': analysis_org_obj.product_id.id,
                            'lot_id': lot_dest_id,
                            'production_id': False,
                            'analysis_template_id': analysis_org_obj.analysis_template_id.id,
                            #hdcm 2013_09_13 v604 'revision': analysis_org_obj.revision,
                            'type_analysis': analysis_org_obj.type_analysis,
                            'analysis_line_ids' : [],
                            'conform': analysis_org_obj.conform,
                            'date_create': analysis_org_obj.date_create,
                            'confirm_by_id': analysis_org_obj.confirm_by_id.id,
                            'date_start': analysis_org_obj.date_start,
                            'approver_id': analysis_org_obj.approver_id.id,
                            'date_end': analysis_org_obj.date_end,
                            'note' : analysis_org_obj.note,
                            'state': analysis_org_obj.state
                             })
            # écrire la ref de la fab
            if fab:
                self.write(cr, uid, analysis_new_id, {'production_id': analysis_org_obj.production_id.id})
            # ajouter le resultats des tests aux lignes de cette analyse
            #liste des tests de l'analyse d'origine
            analysis_org_lines = [x.id for x in analysis_org_obj.analysis_line_ids]
            #print "analysis_org_lines ",analysis_org_lines
            #liste des tests de la nouvelle analyse
            analysis_new_lines = [x.id for x in self.browse(cr, uid, analysis_new_id).analysis_line_ids]
            #print "analysis_new_lines ",analysis_new_lines
            while analysis_org_lines:
                analysis_org_line_obj = self.pool.get('labo.analysis.line').browse(cr, uid, analysis_org_lines.pop())
                #print "analysis_org_line_obj ",analysis_org_line_obj
                self.pool.get('labo.analysis.line').write(cr, uid, analysis_new_lines.pop(),{
                            'test_result': analysis_org_line_obj.test_result,
                            'conform': analysis_org_line_obj.conform,
                            'date_start': analysis_org_line_obj.date_start,
                            'date_end': analysis_org_line_obj.date_end,
                            'lab_tech_id': analysis_org_line_obj.lab_tech_id.id,
                            'date_control': analysis_org_line_obj.date_control,
                            'controler_id': analysis_org_line_obj.controler_id.id,
                            'note' : analysis_org_line_obj.note,
                            'state': analysis_org_line_obj.state
                            })

            wf_service = netsvc.LocalService("workflow")
            # workflow de analysis_new_id dans l'etat 'in_progress'
            wf_service.trg_validate(uid, 'labo.analysis', analysis_new_id, 'button_confirm', cr)
            # workflow des analysis_new_lines
            for analysis_line_obj in self.browse(cr, uid, analysis_new_id).analysis_line_ids:
                wf_service.trg_validate(uid, 'labo.analysis.line', analysis_line_obj.id, 'button_start_test', cr)
                wf_service.trg_validate(uid, 'labo.analysis.line', analysis_line_obj.id, 'button_valid_lab_tech', cr)
                wf_service.trg_validate(uid, 'labo.analysis.line', analysis_line_obj.id, 'button_valid_controler', cr)
            # workflow de analysis_new_id dans l'etat done
            wf_service.trg_validate(uid, 'labo.analysis', analysis_new_id, 'button_done', cr)
        #print "***copy_labo_analysis(labo_analysis)"
        return True


labo_analysis()

# Definition de la classe lignes d'analyse pour un lot
class labo_analysis_line(osv.osv):
    _description='Analysis Line'
    _name = 'labo.analysis.line'
    _columns = {
        'labo_analysis_id': fields.many2one('labo.analysis', 'Analysis', required=True, ondelete='cascade'),
        'product_id': fields.many2one('product.product', 'Product', readonly=True),
        'lot_id': fields.many2one('stock.production.lot', 'Lot', readonly=True),
        'production_id': fields.many2one('mrp.production', 'Production', readonly=True),
        'code_analysis': fields.char('Code analysis', size=16, readonly=True),
        'sequence': fields.integer('Sequence', readonly=True),
        'test_id' : fields.many2one('labo.test', 'Test', readonly=True),
        'name': fields.char('Test', size=256, required=True, readonly=True),
        'specification_range': fields.text('Specification range', required=True, readonly=True),
        'expres_id': fields.many2one('labo.test.express.res', 'Expression of results', readonly=True),
        'test_result': fields.text('Test result', readonly=True, states={'in_progress': [('readonly', False)]}),
        'conform': fields.boolean('Conform', readonly=True, states={'in_progress': [('readonly', False)]}),
        'test_category_id': fields.many2one('labo.test.category', 'Test category', readonly=True),
        'labo_test_method_ids': fields.many2many('labo.test.method', 'labo_analysis_method_rel', 'labo_analysis_line_id','labo_test_method_id','Test methods', readonly=True),
        'date_start': fields.datetime('Start date', readonly=True),
        'date_end': fields.datetime('End date', readonly=True),
        'lab_tech_id': fields.many2one('res.users', 'Laboratory technician', readonly=True),
        'date_control': fields.datetime('Date control', readonly=True),
        'controler_id': fields.many2one('res.users', 'Controler', readonly=True),
        'note' : fields.text('Note', readonly=True, states={'in_progress': [('readonly', False)]}),
        'state': fields.selection([('draft','Draft'),('in_progress','In progress'),
                                  ('valid_lab_tech','Validated by technician'),
                                  ('valid_controler','Validated by controler'),
                                  ('done','Done'),
                                  ('cancel','Cancel')],'Status', readonly=True),
    }
    _order = 'sequence, id'

    _defaults = {
        'state': lambda *a: 'draft',
        'conform': lambda *a: 0,
        'test_result': lambda *a: '',
    }

    def button_start_test(self, cr, uid, ids):
        if ids:
            # verifier que labo_analysis est dans l'etat 'In progress'
            if self.browse(cr, uid, ids[0]).labo_analysis_id.state != 'in_progress':
                raise osv.except_osv(_('Invalid action !'), _('This Analysis is not in state \'In progress\' !'))
            else:
                self.write(cr, uid, ids, {'lab_tech_id': uid,'date_start': time.strftime('%Y-%m-%d %H:%M:%S'),'date_end': False,
                                'controler_id': False,'date_control': False,
                                'state': 'in_progress'})
                return True
        else:
            return False

    def button_valid_lab_tech(self, cr, uid, ids):
        if ids:
            self.write(cr, uid, ids, {'lab_tech_id': uid,'date_end': time.strftime('%Y-%m-%d %H:%M:%S'),
                                      'state': 'valid_lab_tech'})
            return True
        else:
            return False

    def button_valid_controler(self, cr, uid, ids):
        if ids:
            self.write(cr, uid, ids, {'controler_id': uid,'date_control': time.strftime('%Y-%m-%d %H:%M:%S'),
                                      'state': 'valid_controler'})
            return True
        else:
            return False

    def button_in_progress(self, cr, uid, ids):
        if ids:
            # verifier que labo_analysis est dans l'etat 'In progress'
            if self.browse(cr, uid, ids[0]).labo_analysis_id.state != 'in_progress':
                raise osv.except_osv(_('Invalid action !'), _('This Analysis is not in state \'In progress\' !'))
            else:
                self.write(cr, uid, ids, {'lab_tech_id': uid,'date_start': time.strftime('%Y-%m-%d %H:%M:%S'),'date_end': False,
                                          'controler_id': False,'date_control': False,
                                         'state': 'in_progress'})
                return True
        else:
            return False


labo_analysis_line()



