# -*- coding: utf-8 -*-
##############################################################################
#
#    Module Laboratory for Process Industries (OpenERP)
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 


from openerp.osv import fields, osv



class mrp_production(osv.osv):
    _name = 'mrp.production'
    _inherit = 'mrp.production'
    
    _columns = {
        'hd_mo_analysis_ids': fields.one2many('labo.analysis', 'production_id', 'Analysis', readonly=True, states={'in_production': [('readonly', False)]}),
        'automatic_analysis_copy':fields.boolean('Automatic analysis copy', readonly=True, states={'draft': [('readonly', False)], 'in_production': [('readonly', False)]},
                                                 help="Automatic copy of the latest MO analysis towards lots out"),
    }
    _defaults = {
        'automatic_analysis_copy': lambda *a: 1,
    }

    def copy(self, cr, uid, id, default=None,context=None):
        #print "***copy(mrp_production) - id ",id, " - default ", default, " - context ", context
        if default is None : default = {}
        if context is None : context = {}
        default.update({'hd_mo_analysis_ids': [],})
        #print "/***copy(mrp_production) - id ",id, " - default ", default, " - context ", context
        return super(mrp_production, self).copy(cr, uid, id, default, context)

    def action_production_end(self, cr, uid, ids, context=None):
        #print "***action_production_end(mrp_production) - ids ",ids
        if context is None: context = {}
        super(mrp_production, self).action_production_end(cr, uid, ids, context=context)
        # recopie de la derniere analyse (en état 'done') de cet MO vers le ou les lots de sortie
        # si le champ 'automatic_analysis_copy' est True
        production = self.pool.get('mrp.production').browse(cr, uid, ids[0], context=context)
        if production.automatic_analysis_copy:
            # trouver la derniere analyse effectuee avant la fin du MO ayant un etat 'done'
            analysis_mo_ids = [x.id for x in production.hd_mo_analysis_ids]
            while analysis_mo_ids:
                analysis_mo_id = analysis_mo_ids.pop()
                if self.pool.get('labo.analysis').browse(cr, uid, analysis_mo_id).state == 'done':
                    #print "analysis_mo_id ",analysis_mo_id
                    # creer pour chaque lot de sortie une analyse identique
                    self.pool.get('labo.analysis').copy_labo_analysis(cr, uid, analysis_mo_id, [x.id for x in production.hd_mo_lots_ids_fab], fab=True)
                    break
        #print "/***action_production_end(mrp_production)"
        return True


#!!! mrp_production()
