# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 


from osv import fields
from osv import osv
#hdcm 20130704 import ir

import netsvc
from datetime import datetime
import time
from mx import DateTime

import string
from tools.translate import _
import tools
import decimal_precision as dp

CAS_MO_NORMAL = {'types': ('in', 'fab_stock', 'split'), 'states': ('conform','alert_conform'), 'location_under_control': 'not_allowable'}

'''hdcm 2013_10_02 def rounding(f, r):
    if not r:
        return f
    return round(f / r) * r'''

class mrp_production(osv.osv):
    _name = 'mrp.production'
    _inherit = 'mrp.production'
    _description = 'Production'
    _columns = {
        'name': fields.char('Name', size=64, required=True, 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'origin': fields.char('Origin', size=64, 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'priority': fields.selection([('0','Not urgent'),('1','Normal'),('2','Urgent'),('3','Very Urgent')], 'Priority', states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'product_id': fields.many2one('product.product', 'Product', required=True, domain=[('type','<>','service')], 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}, change_default=True),
        'product_qty': fields.float('Product Qty', required=True, 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}, 
                                    digits_compute=dp.get_precision('Qty Module Process')),
        'product_uom': fields.many2one('product.uom', 'Product UOM', 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'location_src_id': fields.many2one('stock.location', 'Production Location', required=True, states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]},
            help="Location where the system will look for products used in raw materials."),
        'location_dest_id': fields.many2one('stock.location', 'Finished Products Location', required=True, 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]},
            help="Location where the system will stock the finished products."),    
        'date_planned': fields.datetime('Planned date', required=True, 
                                    states={'in_production':[('readonly',True)], 'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'date_start': fields.datetime('Start Date', states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'date_finished': fields.datetime('End Date', states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'bom_id': fields.many2one('mrp.bom', 'Bill of Material', domain=[('bom_id','=',False)], states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'routing_id': fields.many2one('mrp.routing', string='Routing', on_delete='set null', 
                                      states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}, help="The list of operations (list of work centers) to produce the finished product. The routing is mainly used to compute work center costs during operations and to plan future loads on work centers based on production plannification."),
        'workcenter_lines': fields.one2many('mrp.production.workcenter.line', 'production_id', 'Workcenters Utilisation', states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'move_created_ids': fields.one2many('stock.move', 'production_id', 'Distribution of the quantities manufactured', readonly=True),
        'move_lines': fields.many2many('stock.move', 'mrp_production_move_ids', 'production_id', 'move_id', 'Moves used products', readonly=True),
        'product_lines': fields.one2many('mrp.production.product.line', 'production_id', 'BOM products', readonly=True),
        'hd_picking_ids_spl': fields.one2many('stock.picking', 'hd_production_id_spl', 'Stock picking', help="This is the list of splitting picking list that have been generated for this Manufacturing order"),#hdcm 2012_12_13 suppression de readonly
        'hd_picking_ids_int': fields.one2many('stock.picking', 'hd_production_id_int', 'Internal picking List', help="This is the list of internal picking list that have been generated for this Manufacturing order", readonly=True),
        'hd_product_lines_waiting_ids': fields.one2many('mrp.production.product.line.used', 'production_id_waiting', 'Product lines waiting', states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'hd_product_lines_procur_ids': fields.one2many('mrp.production.product.line.used', 'production_id_procur', 'Procurement product lines', readonly=True),
        'hd_product_lines_used_ids': fields.one2many('mrp.production.product.line.used', 'production_id_used', 'In process product lines', readonly=True),
        'hd_mo_lots_ids_spl': fields.one2many('stock.production.lot', 'hd_production_id_spl', 'Stock lots in production location', readonly=True, help="This is the list of the splitting lots of this Manufacturing order"),
        'hd_mo_lots_ids_int': fields.one2many('stock.production.lot', 'hd_production_id_int', 'Used lots in MO', readonly=True, help="This is the list of the internal lots of this Manufacturing order"),
        'hd_mo_lots_ids_fab': fields.one2many('stock.production.lot', 'hd_production_id_fab', 'MO production lots', readonly=True, help="This is the list of the production lots of this Manufacturing order"),
        'hd_note': fields.text('Notes', states={'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'state': fields.selection([
                        ('draft','Draft'),    
                        ('in_production','In Production'),
                        ('production_end','Production End'),
                        ('done','Done'),
                        ('cancel','Canceled'),
                    ],'State', readonly=True),
        'hd_nb_production_lot': fields.integer('Nb production lots', 
            states={'in_production': [('readonly', True)], 'production_end': [('readonly', True)], 'cancel': [('readonly', True)], 'done': [('readonly', True)]}),
        #hdcm 2013_10_02 voir hd_labo 'hd_mo_analysis_ids': fields.one2many('labo.analysis', 'production_id', 'Analysis', readonly=True, states={'in_production': [('readonly', False)]}),
        #hdcm 2013_10_02 voir hd_labo 'automatic_analysis_copy':fields.boolean('Automatic analysis copy', readonly=True, states={'draft': [('readonly', False)], 'in_production': [('readonly', False)]},
        #hdcm 2013_10_02 voir hd_labo                                          help="Automatic copy of the latest MO analysis towards lots out"),
        'automatic_picking':fields.boolean('Automatic picking', readonly=True, states={'draft': [('readonly', False)], 'in_production': [('readonly', False)]}),
        'company_id': fields.many2one('res.company','Company',required=True, states={'production_end':[('readonly',True)], 'cancel':[('readonly',True)], 'done':[('readonly',True)]}),
        'hd_type_mo': fields.selection([
                        ('standard','Standard'),    
                        ('packaging','Packaging')
                    ],'Type MO', readonly=True, help='Type of Manufacturing Order\nStandard\nPackaging'),
    }
    _defaults = {
        'hd_nb_production_lot': lambda *a: 1,
        #hdcm 2013_10_02 voir hd_labo 'automatic_analysis_copy': lambda *a: 1,
        'automatic_picking': lambda *a: 1,
        'hd_type_mo': lambda *a: 'standard',
    }

#HDcm 2012_03_29 v603_02chem - pour eviter de creer une fab pour un produit non gere par lot
    def create(self, cr, user, vals, context=None):
        #print " ***create(mrp_production) - vals ", vals, " - context ", context
        if 'product_id' in vals:
            product_obj = self.pool.get('product.product')
            product = product_obj.browse(cr, user, vals['product_id'])
            if not product.hd_track_lot:
                raise osv.except_osv(_('Invalid action !'), _('This manufacturing order can not be created because the product (%s) to be manufactured is not managed by lot !')%(product.name,))
        new_id = super(mrp_production, self).create(cr, user, vals, context)
        #print "/***create(mrp_production) - new_id ", new_id
        return new_id

    def copy(self, cr, uid, id, default=None,context=None):
        #print "***copy(mrp_production) - id ",id, " - default ", default, " - context ", context
        #HDcm 2012_03_01 v506_06chem if not default: default = {}
        if default is None : default = {}
        if context is None : context = {}
        default.update({
            'hd_picking_ids_spl': [],
            'hd_picking_ids_int': [],
            'hd_product_lines_procur_ids': [],
            'hd_product_lines_used_ids': [],
            'hd_mo_lots_ids_spl': [],
            'hd_mo_lots_ids_int': [],
            'hd_mo_lots_ids_fab': [],
            'hd_note': '',
            'hd_nb_production_lot': 1,
            'move_prod_id': 0,
            'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
            'date_finnished': 0,
            'bom_id': 0,
            'routing_id': 0,
            'workcenter_lines': [],
            #hdcm 2013_10_02 voir hd_labo 'hd_mo_analysis_ids': [],
            #HDcm 2012_03_01 v603
            'name': self.pool.get('ir.sequence').get(cr, uid, 'mrp.production'),
            'move_lines' : [],
            'move_lines2' : [],
            'move_created_ids' : [],
            'move_created_ids2' : [],
            'product_lines' : [],
            'picking_id': False
            #/HDcm 2012_03_01 v603
        })
        #print "/***copy(mrp_production) - id ",id, " - default ", default, " - context ", context
        return super(mrp_production, self).copy(cr, uid, id, default, context)

    # autoriser unlink MO uniquement pour l'etat draft
    def unlink(self, cr, uid, ids, context=None):
        #print "***unlink(mrp_production) - ids ",ids
        for production in self.browse(cr, uid, ids, context=context):
            #hdcm 2013_08_21 v70 if production.state not in ('draft', 'cancel'):
            if production.state in ['draft']:
                # unlink des lignes mrp_production_product_line_used
                if production.hd_product_lines_waiting_ids:
                    self.pool.get('mrp.production.product.line.used').unlink(cr, uid, [x.id for x in production.hd_product_lines_waiting_ids])
                #HDcm 2012_03_08 v603_01chem
                if production.product_lines:
                    self.pool.get('mrp.production.product.line').unlink(cr, uid, [x.id for x in production.product_lines])
                #/HDcm 2012_03_08 v603_01chem
                unlink_ids.append(production.id)
            else:
                raise osv.except_osv(_('Invalid Action!'), _('Cannot delete a manufacturing order in state \'%s\'.') % production.state)
        #print "/***unlink(mrp_production)"
        return super(mrp_production, self).unlink(cr, uid, ids, context=context)

    # autoriser cancel MO pour l'etat draft et in_production
    def action_cancel(self, cr, uid, ids, context=None):
        #print "***action_cancel(mrp_production) - ids ",ids, " - context ", context
        if context is None: context = {}
        for production in self.browse(cr, uid, ids):
            if production.state in ['draft','in_production']:
                # cancel des lignes mrp_production_product_line_used
                if production.hd_product_lines_used_ids:
                    self.pool.get('mrp.production.product.line.used').button_cancel(cr, uid, [x.id for x in production.hd_product_lines_used_ids])
                if production.hd_product_lines_waiting_ids:
                    self.pool.get('mrp.production.product.line.used').button_cancel(cr, uid, [x.id for x in production.hd_product_lines_waiting_ids])
                self.write(cr, uid, [production.id], {'state': 'cancel'})
            else:
                raise osv.except_osv(_('Invalid action !'), _('Cannot cancel a Production Order which is %s !')%(production.state,))
        #print "/***action_cancel(mrp_production)"
        return super(mrp_production, self).action_cancel(cr, uid, ids, context=context)

    # interdire un MO si le produit fabriqué n'est pas gere par lot
    def write(self, cr, uid, ids,vals,context={}):
        #print "***write(mrp_production) - ids ",ids," - vals ",vals," - context ",context
        if 'product_id' in vals:
            product_obj = self.pool.get('product.product').browse(cr, uid, vals['product_id'])
            if not product_obj.hd_track_lot:
                raise osv.except_osv(_('Invalid action !'), _('To create this production order, \'Track Process Lots\' of product \'%s\' must be checked.') %(product_obj.name,))
        return super(mrp_production,self).write(cr, uid, ids, vals, context)
    
    def test_if_product(self, cr, uid, ids):
        """
        @return: True or False
        """
        res = True
        '''for production in self.browse(cr, uid, ids):
            if not production.product_lines:
                if not self.action_compute(cr, uid, [production.id]):
                    res = False'''
        #print "res : ",res
        return res

    def action_compute(self, cr, uid, ids, properties=None, context=None):
        """ Computes bills of material of a product.
        @param properties: List containing dictionaries of properties.
        @return: No. of products.
        """
        #print "***action_compute(mrp_production) - ids ",ids," - properties ",properties, " - context ", context
        if context is None: context = {}
        if properties is None:
            properties = []
        results = []
        bom_obj = self.pool.get('mrp.bom')
        uom_obj = self.pool.get('product.uom')
        prod_line_obj = self.pool.get('mrp.production.product.line')
        workcenter_line_obj = self.pool.get('mrp.production.workcenter.line')
        for production in self.browse(cr, uid, ids):
            cr.execute('delete from mrp_production_product_line where production_id=%s', (production.id,))
            cr.execute('delete from mrp_production_workcenter_line where production_id=%s', (production.id,))
            #hdcm
            cr.execute('delete from mrp_production_product_line_used where production_id_waiting=%s', (production.id,))
            #/hdcm
            bom_point = production.bom_id
            bom_id = production.bom_id.id
            if not bom_point:
                bom_id = bom_obj._bom_find(cr, uid, production.product_id.id, production.product_uom.id, properties)
                if bom_id:
                    bom_point = bom_obj.browse(cr, uid, bom_id)
                    routing_id = bom_point.routing_id.id or False
                    self.write(cr, uid, [production.id], {'bom_id': bom_id, 'routing_id': routing_id})

            if not bom_id:
                raise osv.except_osv(_('Error!'), _("Cannot find a bill of material for this product."))
            factor = uom_obj._compute_qty(cr, uid, production.product_uom.id, production.product_qty, bom_point.product_uom.id)
            res = bom_obj._bom_explode(cr, uid, bom_point, factor / bom_point.product_qty, properties, routing_id=production.routing_id.id)
            results = res[0]
            results2 = res[1]
            for line in results:
                line['production_id'] = production.id
                prod_line_obj.create(cr, uid, line)
                #hdcm
                line_used={
                    'name':line['name'],
                    'product_id':line['product_id'],
                    'qty_bom':line['product_qty'],
                    'qty_lab':line['product_qty'],
                    'qty_pick':line['product_qty'],
                    'qty_prod':line['product_qty'],
                    'product_uom':line['product_uom'],
                    'procure_method':self.pool.get('product.product').browse(cr, uid, line['product_id']).procure_method,
                    'supply_method':self.pool.get('product.product').browse(cr, uid, line['product_id']).supply_method,
                    'production_id_waiting':production.id}
                self.pool.get('mrp.production.product.line.used').create(cr, uid, line_used)
                #/hdcm
            for line in results2:
                line['production_id'] = production.id
                workcenter_line_obj.create(cr, uid, line)
        #print "/***action_compute(mrp_production) - len(results) ",len(results)
        return len(results)

    def action_in_production(self, cr, uid, ids):
        """ Changes state to In Production and writes starting date.
        @return: True
        """
        #print "***action_in_production(mrp.production) - ids : ",ids
        move_obj = self.pool.get('stock.move')#HDcm 2012_0412 604_02chem
        product_uom_obj = self.pool.get('product.uom')#HDcm 2012_0412 604_02chem
        picking_id=False
        for production in self.browse(cr, uid, ids):
            if not production.hd_product_lines_waiting_ids:
                raise osv.except_osv(_('Error'), _('No waiting product lines'))
            self.write(cr, uid, [production.id], {'date_start': time.strftime('%Y-%m-%d %H:%M:%S')}) #hdcm 2009_12_02 initialisation de date_start           
            if production.move_prod_id:
                self.pool.get('stock.move').write(cr, uid, [production.move_prod_id.id], {'location_id':production.location_dest_id.id})
            # creation du lot fabrique au debut du MO
            # creation d'un lot correspondant a ce move, 
            # pour les produits dont l'indicateur hd_track_lot est activé
            moves_procur=[]
            for move in production.move_created_ids:
                move = self.pool.get('stock.move').browse(cr, uid, [move.id])[0]
                # faire le test pour savoir si la fab vient d'un SO
                type=''
                doc_org_id=0
                doc_inter_id=0
                if move.move_dest_id.id and move.move_dest_id.sale_line_id.id:
                    type='sale_fab'
                    # doc_org_id et doc_inter_id
                    doc_org=self.pool.get('sale.order.line').browse(cr,uid,move.move_dest_id.sale_line_id.id)
                    doc_org_id=doc_org.order_id.id
                    doc_inter_id=self.pool.get('stock.move').browse(cr,uid,move.id).production_id.id
                else:
                    type='fab_stock'
                    doc_org_id=self.pool.get('stock.move').browse(cr,uid,move.id).production_id.id
                    moves_procur.append(move.move_dest_id.id)
                # le paramètre context est utilisé pour l'initialisation des 4 champs date péremption
                # par la fonction calc_date de la classe stock.production.lot
                #print "creation lot"
                hd_qty_org = product_uom_obj._compute_qty(cr, uid, move.product_uom.id, move.product_qty, to_uom_id=move.product_id.uom_id.id)#HDcm 2012_04_14 604_02chem 
                lot_id = self.pool.get('stock.production.lot').create(cr, uid, {
                                        #hdcm 2013_09_16 v604 'name':'FAB[%s][%s]' %(move.product_id.code,time.strftime('%Y%m%d%H%M')),
                                        'name':'temporary_name',
                                        'hd_production_id_fab':production.id,
                                        'product_id':move.product_id.id,
                                        'hd_product_uom_id':move.product_id.uom_id.id,
                                        #HDcm 2012_04_14 506_06chem 'hd_qty_org':move.product_qty,
                                        'hd_qty_org': hd_qty_org,
                                        'hd_indtrack':True,
                                        'hd_type':type,
                                        'hd_doc_org_id':doc_org_id,
                                        'hd_doc_inter_id':doc_inter_id,
                                        'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                                        },context= {'product_id':move.product_id.id})
                # compléter le name du lot avec son id
                lot_obj=self.pool.get('stock.production.lot')
                #hdcm 2013_09_16 v604 self.pool.get('stock.production.lot').write(cr,uid, [lot_id], {'name':'%s:%07d'%(lot_obj.name,lot_id)})
                lot_obj.write(cr,uid, [lot_id], {'name':lot_obj.get_name_lot(cr, uid, 'FAB', move.product_id, lot_id)})
                # si fab pour une vente, renseigner hd_reserv et hd_reserv_partner_id
                if type=='sale_fab':
                    self.pool.get('stock.production.lot').write(cr,uid, [lot_id], {
                                    'hd_reserv':'vente',
                                    'hd_reserv_partner_id':doc_org.order_id.partner_id.id,
                                    })
                # si fab pour une fab, renseigner hd_reserv et hd_reserv_production_id
                if type=='fab_stock' and (move.move_dest_id.id):
                    # --- retrouver les mrp.production.product.line.used qui ont pour move_id_spl=move.move_dest_id.id 
                    # et mettre state à 'procur_done'
                    mrp_production_product_line_used_id_list = self.pool.get('mrp.production.product.line.used').search(cr, uid, [('move_id_spl', '=', move.move_dest_id.id)])
                    #print "mrp_production_product_line_used_id_list : ",mrp_production_product_line_used_id_list
                    for mrp_production_product_line_used_obj in self.pool.get('mrp.production.product.line.used').browse(cr,uid,mrp_production_product_line_used_id_list):
                        if mrp_production_product_line_used_obj.state == 'procur_run':
                            self.pool.get('mrp.production.product.line.used').write(cr, uid, [mrp_production_product_line_used_obj.id], {'state':'procur_done'})
                    # vérifier si cette fab est issue d'une autre fab
                    # retrouver le procurement d'origine
                    if production.move_prod_id.id:
                        # chercher le procurement dont le move_id=move.move_prod_id.id
                        proc_id=self.pool.get('procurement.order').search(cr, uid, [('move_id', '=', production.move_prod_id.id)])[0]
                        # retrouver la production d'origine
                        proc_org=self.pool.get('procurement.order').browse(cr,uid,proc_id)
                        if proc_org.hd_production_id.id:
                            self.pool.get('stock.production.lot').write(cr,uid, [lot_id], {
                                    'hd_reserv':'interne',
                                    'hd_reserv_production_id':proc_org.hd_production_id.id,
                                    #HDcm 2012_03_14 v506_06chem 'hd_production_id_spl':proc_org.hd_production_id.id,
                                    })            
                # ecrire la reference du lot dans le move
                self.pool.get('stock.move').write(cr,uid, [move.id], {'prodlot_id':lot_id, 'hd_prodlot_id':lot_id})
            #/hdcm 2010_04_12 creation du lot fabrique au debut du MO
            
            #HDcm 2012_02_29 vOE self.write(cr, uid, ids, {'state': 'in_production', 'date_start': time.strftime('%Y-%m-%d %H:%M:%S')})
            self.write(cr, uid, [production.id], {'state': 'in_production', 'date_start': time.strftime('%Y-%m-%d %H:%M:%S')})
            #hdcm2010_04_16 faire avancer le WF (button_mo_start)
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'mrp.production', production.id, 'button_mo_start', cr)
            #/hdcm2010_04_16 faire avancer le WF (button_mo_start)
        #print "/***action_in_production(mrp.production)"
        return True

    def button_confirm_all_picking(self, cr, uid, ids, context=None, properties=[]):
        #print "***button_confirm_all_picking(mrp.production) - ids : ",ids, " - context ", context, " - properties ", properties
        #print time.clock()
        if context is None: context = {}
        for production in self.browse(cr, uid, ids):
            wf_service = netsvc.LocalService("workflow")
            for picking_obj in production.hd_picking_ids_spl:
                if picking_obj.state == 'draft':
                    #print "WF picking - validate button confirm"
                    wf_service.trg_validate(uid, 'stock.picking', picking_obj.id, 'button_confirm', cr)
                if (picking_obj.state == 'confirmed'):
                    self.pool.get('stock.picking').action_assign(cr, uid, [picking_obj.id])
        #print time.clock()
        #print "/***button_confirm_all_picking(mrp.production)"
        return True

    def button_force_confirm_all_picking(self, cr, uid, ids, properties=[]):
        #print "***button_force_confirm_all_picking(mrp.production)"
        for production in self.browse(cr, uid, ids):
            wf_service = netsvc.LocalService("workflow")
            for splitting_obj in production.hd_picking_ids_spl:
                wf_service.trg_validate(uid, 'stock.picking', splitting_obj.id, 'button_confirm', cr)
                self.pool.get('stock.picking').force_assign(cr, uid, [splitting_obj.id])
        return True

    def button_make_all_picking(self, cr, uid, ids, context=None, properties=[]):
        #print " ***button_make_all_picking(mrp.production)"
        #print time.clock()
        if context is None: context = {}
        for production in self.browse(cr, uid, ids):
            wf_service = netsvc.LocalService("workflow")
            for splitting_obj in production.hd_picking_ids_spl:
                #print "splitting_obj ",splitting_obj
                wf_service.trg_validate(uid, 'stock.picking', splitting_obj.id, 'button_done', cr)
        #print time.clock()
        #print "/***button_make_all_picking(mrp.production)"
        return True

    def button_add_to_mo(self, cr, uid, ids, context=None, properties=[]):
        #print " ***button_add_to_mo(mrp.production) - ids ", ids, " - context ", context
        #print time.clock()
        if context is None: context = {}
        context['MO_add_to_mo'] = ids#HDcm 2012_03_15 v603_02chem
        self.action_confirm(cr, uid, ids, context=context)
        for production in self.browse(cr, uid, ids):
            if production.automatic_picking:
                self.button_confirm_all_picking(cr, uid, [production.id], context=context)
                self.button_make_all_picking(cr, uid, [production.id], context=context)
                self.action_confirm(cr, uid, [production.id], context=context)
        #print time.clock()
        #print "/***button_add_to_mo(mrp.production)"
        return True

    def action_confirm(self, cr, uid, ids, context=None):#button_add_to_mo
        """ Confirms production order.
        @return: Newly generated picking Id.
        """
        #print "***action_confirm(mrp_production) - ids : ",ids, " - context ", context
        if context is None: context = {}
        context['prog'] = 'action_confirm(mrp_production)'#HDcm 2012_03_15 v603_02chem
        #HDcm 2012_02_29 v603_01chem
        seq_obj = self.pool.get('ir.sequence')
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        proc_obj = self.pool.get('procurement.order')
        mo_product_line_used_obj = self.pool.get('mrp.production.product.line.used')
        lot_obj = self.pool.get('stock.production.lot')
        wf_service = netsvc.LocalService("workflow")
        #/HDcm 2012_02_29 v603_01chem
        for production in self.browse(cr, uid, ids):
            #HDcm 2012_02_29 v603_01chem routing_loc = None
            #HDcm 2012_02_29 v603_01chem pick_type = 'internal'
            #hdcm 2013_08_21 v604 address_id = False
            partner_id = False
            if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
                #HDcm 2012_02_29 v603_01chem routing_loc = production.bom_id.routing_id.location_id
                #HDcm 2012_02_29 v603_01chem if routing_loc.usage<>'internal':
                    #HDcm 2012_02_29 v603_01chem pick_type = 'out'
                #hdcm 2013_08_21 v604 address_id = routing_loc.address_id and routing_loc.address_id.id or False
                partner_id = routing_loc.partner_id and routing_loc.partner_id.id or False
                #HDcm 2012_02_29 v603_01chem routing_loc = routing_loc.id
            #----------------
            # 1er TRAITEMENT
            #----------------
            # traiter les lignes 'waiting product lines' si ligne sélectionnée ET state ='draft' ET qty_pick > 0
            # ET la ligne est en 'Make to stock'
            todo_waiting_product_lines = []
            for waiting_product_lines in production.hd_product_lines_waiting_ids:

                #hdcm 2009_12_29 calculer les qtes en fonction de l'uom du produit
                qty_prod_corr=self.pool.get('product.uom')._compute_qty(cr, uid, waiting_product_lines.product_uom.id, 
                                                                            waiting_product_lines.qty_prod, 
                                                                            to_uom_id=waiting_product_lines.product_id.uom_id.id)
                qty_pick_corr=self.pool.get('product.uom')._compute_qty(cr, uid, waiting_product_lines.product_uom.id, 
                                                                            waiting_product_lines.qty_pick, 
                                                                            to_uom_id=waiting_product_lines.product_id.uom_id.id)
                qty_bom_corr=self.pool.get('product.uom')._compute_qty(cr, uid, waiting_product_lines.product_uom.id, 
                                                                            waiting_product_lines.qty_bom, 
                                                                            to_uom_id=waiting_product_lines.product_id.uom_id.id)
                qty_lab_corr=self.pool.get('product.uom')._compute_qty(cr, uid, waiting_product_lines.product_uom.id, 
                                                                            waiting_product_lines.qty_lab, 
                                                                            to_uom_id=waiting_product_lines.product_id.uom_id.id)
                product_uom_def=waiting_product_lines.product_id.uom_id.id
                self.pool.get('mrp.production.product.line.used').write(cr, uid, [waiting_product_lines.id], 
                                                                            {'qty_prod':qty_prod_corr,
                                                                             'qty_pick':qty_pick_corr,
                                                                             'qty_bom':qty_bom_corr,
                                                                             'qty_lab':qty_lab_corr,
                                                                             'product_uom':product_uom_def
                                                                             })
                #print "1er TRAITEMENT"
                # if waiting_product_lines.sel and (waiting_product_lines.qty_pick>0) and (waiting_product_lines.product_id.type in ('product', 'consu')):
                if waiting_product_lines.sel and (waiting_product_lines.qty_pick>0) \
                   and (waiting_product_lines.state == 'draft') and (waiting_product_lines.product_id.type in ('product', 'consu')):
                    todo_waiting_product_lines.append(waiting_product_lines.id)
                    # pour les lignes ayant 'procure_method'='make_to_stock'
                    #if waiting_product_lines.procure_method=='make_to_stock':
                    if True:
                        # creation du picking,traiter les 2 cas : produit géré par lot et produit non géré par lot
                        type_picking = ''
                        if waiting_product_lines.product_id.hd_track_lot:# si produit géré par lot, creer un picking de type 'split'
                            type_picking = 'split'
                            pick_name = seq_obj.get(cr, uid, 'stock.picking.split')
                        else: # sinon créer un picking de type 'internal'
                            type_picking = 'internal'
                            pick_name = seq_obj.get(cr, uid, 'stock.picking')#HDcm 2013_10_03
                        picking_id = self.pool.get('stock.picking').create(cr, uid, {
                            'name': pick_name,#HDcm 2012_02_29 v603_01chem
                            'origin': (production.origin or '').split(':')[0] +':'+production.name,
                            'type': type_picking,
                            'move_type': 'one',
                            'state': 'draft',
                            'auto_picking': False,
                            'hd_production_id_spl': production.id,
                            #hdcm 2013_08_2 v604 'address_id': address_id,
                            'partner_id': partner_id,
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                        })
                        # création du move pour ce picking
                        #hdcm 2013_02_26 lorsque le lot est imposé, la location_id doit etre celle du lot
                        var_location_id = production.location_src_id.id
                        if waiting_product_lines.lot_org:
                            var_location_id = lot_obj.browse(cr, uid , waiting_product_lines.lot_org.id).hd_location_id.id
                        # calculer les qtes en fonction de l'uom du produit
                        move_id = self.pool.get('stock.move').create(cr, uid, {
                            'name':'PROD:'+production.name,
                            'picking_id':picking_id,
                            'product_id': waiting_product_lines.product_id.id,
                            'product_qty': qty_pick_corr,
                            'product_uom': product_uom_def,
                            'date_planned': production.date_planned,
                            'location_id': var_location_id,
                            'location_dest_id': production.product_id.product_tmpl_id.property_stock_production.id,
                            'state': 'draft',
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                            'hd_lot_org': waiting_product_lines.lot_org and waiting_product_lines.lot_org.id#HDcm 2012_05_30 v604_03chem
                        })
                        #print "creation move 1 ", move_id
                        # écrire l'id de ce move dans waiting_product_lines
                        self.pool.get('mrp.production.product.line.used').write(cr, uid, [waiting_product_lines.id], {'move_id_spl':move_id})
                        # création du procurement
                        proc_id = self.pool.get('procurement.order').create(cr, uid, {
                            'name': production.name+':'+ (production.origin or ''),
                            'origin': production.name+':'+ (production.origin or ''),
                            'date_planned': production.date_planned,
                            'product_id': waiting_product_lines.product_id.id,
                            'product_qty': waiting_product_lines.qty_pick,
                            'product_uom': waiting_product_lines.product_uom.id,
                            'location_id': production.location_src_id.id,
                            'procure_method': waiting_product_lines.procure_method,
                            'hd_production_id':production.id,
                            'production_id':production.id,
                            'move_id': move_id, 
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                        })
                        wf_service = netsvc.LocalService("workflow")
                        wf_service.trg_validate(uid, 'procurement.order', proc_id, 'button_confirm', cr)
            #print "todo_waiting_product_lines : ",todo_waiting_product_lines
            if len(todo_waiting_product_lines):
                self.pool.get('mrp.production.product.line.used').write(cr, uid, todo_waiting_product_lines, {'production_id_waiting':False,'production_id_procur':production.id,'state':'procur_run'})
            #-----------------
            # 2eme TRAITEMENT
            #-----------------
            todo_used_product_lines = []

            # prendre en compte les lignes 'procur product lines' si qty_prod > 0
            for procur_product_lines in production.hd_product_lines_procur_ids:
                #print "2eme TRAITEMENT"
                if procur_product_lines.qty_prod>0:
                    # dupliquer cette ligne
                    # --- mettre qty_pick=0 dans la ligne dupliquée
                    used_product_lines_id=self.pool.get('mrp.production.product.line.used').copy(cr, uid, procur_product_lines.id, {'qty_pick': 0.0})
                    # --- mettre qty_bom=0, qty_lab=0 et qty_prod=0 dans la 1ère ligne
                    self.pool.get('mrp.production.product.line.used').write(cr, uid, [procur_product_lines.id], {'qty_bom':0.0,'qty_lab':0.0,'qty_prod':0.0})
                    todo_used_product_lines.append(used_product_lines_id)
                
            # prendre en compte les lignes 'waiting product lines' si ligne sélectionnée ET state ='draft' ET qty_pick <= 0 ET qty_prod>0        
            for waiting_product_lines in production.hd_product_lines_waiting_ids:
                if waiting_product_lines.sel and (waiting_product_lines.qty_pick<=0) and (waiting_product_lines.qty_prod>0) and (waiting_product_lines.product_id.type in ('product', 'consu')) :
                    todo_used_product_lines.append(waiting_product_lines.id)
            # prendre en compte les lignes 'used product lines' si 'state'='waiting_product'
            for used_product_lines in production.hd_product_lines_used_ids:
                if used_product_lines.state=='waiting_product':
                    todo_used_product_lines.append(used_product_lines.id)
            #print "todo_used_product_lines : ",todo_used_product_lines
            # traitement    
            if len(todo_used_product_lines):
                mo_product_line_used_obj.write(cr, uid, todo_used_product_lines, {
                                'production_id_waiting':False,
                                'production_id_procur':False,
                                'production_id_used':production.id,
                                'state':'waiting_product'})   
                # établir une liste des produits geres et une liste des produits non gérés par lot
                todo_used_product_lines_managed = []
                todo_used_product_lines_not_managed = []
                for used_product_lines in mo_product_line_used_obj.browse(cr, uid, todo_used_product_lines):
                    if used_product_lines.product_id.hd_track_lot:
                        todo_used_product_lines_managed.append(used_product_lines.id)
                    else:
                        todo_used_product_lines_not_managed.append(used_product_lines.id)
                # cas des produits geres par lots
                #print "cas des produits geres par lots"
                #print "todo_used_product_lines_managed ", todo_used_product_lines_managed
                picking_id=False
                toconfirm = False
                moves_prod = []
                used_product_lines_list = []
                for used_product_lines in mo_product_line_used_obj.browse(cr, uid, todo_used_product_lines_managed):
                    # chercher un ou des lots split de ce MO pour prélever la qty_prod de cette ligne
                    lot_find_list = self._get_lot_spl_mo(cr,uid,[production.id],used_product_lines.id, context=context)
                    #print "lot_find_list ", lot_find_list
                    while len (lot_find_list):
                        lot_find = lot_find_list.pop(0)
                        #print "lot_find : ",lot_find
                        # traiter les lignes dont les qty_prod sont dispo
                        source = production.product_id.product_tmpl_id.property_stock_production.id
                        used_product_lines_list.append(lot_find['used_product_lines_id'])
                        if not picking_id:
                            # création d'un picking contenant ce move
                            picking_id = pick_obj.create(cr, uid, {
                                'origin': (production.origin or '') +':'+production.name,
                                'type': 'internal',
                                'move_type': 'one',
                                'state': 'auto',
                                'auto_picking': True,
                                'hd_production_id_int': production.id,
                                'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                                })
                            toconfirm = True    
                        # création d move de l'ordre de production (en imposant le lot d'origine)
                        # --- retrouver l'id du move dont 'production_id'=production.id
                        move_production_id = move_obj.search(cr, uid,[('production_id', '=', production.id)])[0]
                        res_dest_id = move_obj.create(cr, uid, {
                            'name':'PROD:'+production.name,
                            'date_planned': production.date_planned,
                            'product_id': lot_find['product_id'],
                            'product_qty': lot_find['qty_prelev'],
                            'product_uom': lot_find['product_uom'],
                            'location_id': source,
                            'location_dest_id': source,
                            'move_dest_id': move_production_id,
                            'state': 'draft',
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                        })
                        #print "creation move 3 ", res_dest_id
                        moves_prod.append(res_dest_id)
                        cr.execute('insert into mrp_production_move_ids (production_id,move_id) values (%s,%s)', (production.id, res_dest_id))
                        # création du move du picking
                        move_id = move_obj.create(cr, uid, {
                            'name':'PROD:'+production.name,
                            'date_planned': production.date_planned,
                            'picking_id':picking_id,
                            'product_id': lot_find['product_id'],
                            'product_qty': lot_find['qty_prelev'],
                            'product_uom': lot_find['product_uom'],
                            'location_id': source,
                            'location_dest_id': source,
                            'move_dest_id': res_dest_id,
                            'hd_lot_org':lot_find['lot_find_id'],
                            'state': 'waiting',#HDcm Attention ne pas mettre en draft
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                        })
                        move_obj.check_assign(cr, uid, [move_id])
                # cas des produits non geres
                #print "cas des produits non geres"
                toconfirm2=False
                moves_prod2=[]
                if len(todo_used_product_lines_not_managed):
                    #si le move correspondant est dans l'état 'done', retirer ce used_product_lines de la liste todo_used_product_lines_not_managed
                    #print "todo_used_product_lines_not_managed ",todo_used_product_lines_not_managed
                    todo_trav = []
                    message_warning = ''
                    for used_product_lines_not_managed in todo_used_product_lines_not_managed:
                        qty_remaining = self._get_product_remaining_mo(cr,uid,[production.id],used_product_lines_not_managed)
                        qty_requested = mo_product_line_used_obj.browse(cr, uid, used_product_lines_not_managed).qty_prod
                        #print "qty_remaining ",qty_remaining
                        if qty_remaining >= qty_requested:
                            todo_trav.append(used_product_lines_not_managed)
                        else:# prevenir le user
                            message_warning = 'product "%s" -- qty requested: %f - Remainning qty : %f' %(self.pool.get('mrp.production.product.line.used').browse(cr, uid, used_product_lines_not_managed).product_id.name,\
                                   qty_requested,qty_remaining,)
                            #print "message_warning ",message_warning
                    todo_used_product_lines_not_managed = todo_trav
                    #print "todo_used_product_lines_not_managed ",todo_used_product_lines_not_managed
                    picking_id2=False
                    for used_product_lines in mo_product_line_used_obj.browse(cr, uid, todo_used_product_lines_not_managed):
                        if not picking_id2:
                            # création d'un picking contenant tous ces moves
                            picking_id2 = self.pool.get('stock.picking').create(cr, uid, {
                                'origin': (production.origin or '') +':'+production.name,
                                'type': 'internal',
                                'move_type': 'one',
                                'state': 'auto',
                                'auto_picking': True,
                                'hd_production_id_int': production.id,
                                'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                                })
                            toconfirm2=True 
                        # création des moves de l'ordre de production
                        # --- retrouver l'id du move dont 'production_id'=production.id
                        move_production_id2 = move_obj.search(cr, uid,[('production_id', '=', production.id)])[0]
                        res_dest_id2 = move_obj.create(cr, uid, {
                            'name':'PROD:'+production.name,
                            'date_planned': production.date_planned,
                            'product_id': used_product_lines.product_id.id,
                            'product_qty': used_product_lines.qty_prod,
                            'product_uom': used_product_lines.product_id.product_tmpl_id.uom_id.id,
                            'location_id': production.product_id.product_tmpl_id.property_stock_production.id,
                            'location_dest_id': production.product_id.product_tmpl_id.property_stock_production.id,
                            'move_dest_id': move_production_id2,
                            'state': 'draft',
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                        })
                        #print "creation move 5 ", res_dest_id2
                        moves_prod2.append(res_dest_id2)
                        cr.execute('insert into mrp_production_move_ids (production_id,move_id) values (%s,%s)', (production.id, res_dest_id2))
                        # création des moves du picking
                        move_id2 = move_obj.create(cr, uid, {
                            'name':'PROD:'+production.name,
                            'date_planned': production.date_planned,
                            'picking_id':picking_id2,
                            'product_id': used_product_lines.product_id.id,
                            'product_qty': used_product_lines.qty_prod,
                            'product_uom': used_product_lines.product_id.product_tmpl_id.uom_id.id,
                            'location_id': production.product_id.product_tmpl_id.property_stock_production.id,
                            'location_dest_id': production.product_id.product_tmpl_id.property_stock_production.id,
                            'move_dest_id': res_dest_id2,
                            'state': 'draft',
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                            })
                        move_obj.check_assign(cr, uid, [move_id2])
                if toconfirm:# produits gere
                    #print "produits gere"
                    wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
                    context['prog'] = 'action_confirm(mrp_production)'#HDcm 2012_04_16 v604_02chem
                    pick_obj.action_assign(cr, uid, [picking_id], context=context)
                if toconfirm2:# produits non geres
                    #print "roduits non geres"
                    wf_service.trg_validate(uid, 'stock.picking', picking_id2, 'button_confirm', cr)
                    pick_obj.action_assign(cr, uid, [picking_id2])
                # passer les moves de l'ordre de production dans l'état done
                #print "move vers done"
                move_obj.action_done(cr, uid, moves_prod, context=context)
                move_obj.action_done(cr, uid, moves_prod2, context=context)
                #print "passer les lignes de todo_used_product_lines_not_managed à l'état 'used'"
                # passer les lignes de todo_used_product_lines_not_managed à l'état 'used'
                mo_product_line_used_obj.write(cr, uid, used_product_lines_list, {'state':'used'})
                mo_product_line_used_obj.write(cr, uid, todo_used_product_lines_not_managed, {'state':'used'})
            #HDcm 2012_02_29 v603_01chem
            message = _("Manufacturing order '%s' is scheduled for the %s.") % (
                production.name,
                datetime.strptime(production.date_planned,'%Y-%m-%d %H:%M:%S').strftime('%m/%d/%Y'),
            )
            self.log(cr, uid, production.id, message)
            #/HDcm 2012_02_29 v603_01chem
        #print "/***action_confirm(mrp_production)"
        return True

            
    # cas des produits geres par lots
    # cherche pour une ligne donnée, les lots MO qui peuvent être utilisés
    # argument : id de la ligne traitée
    # return : liste de dictionnaires
    def _get_lot_spl_mo(self, cr, uid, ids, used_product_lines_id, context=None):
        #print "***_get_lot_spl_mo(mrp_production) - ids ",ids," - used_product_lines_id ",used_product_lines_id, " - context ", context
        if context is None : context = {}
        lot_obj = self.pool.get('stock.production.lot')
        mo_product_line_used_obj = self.pool.get('mrp.production.product.line.used')
        res=[]
        for production in self.browse(cr,uid,ids):
            used_product_line = mo_product_line_used_obj.browse(cr,uid,used_product_lines_id)
            qty_rest_a_prelev = used_product_line.qty_prod
            for lot_spl_id in lot_obj.search(cr, uid, [
                                        ('hd_production_id_spl', '=', production.id), 
                                        ('product_id','=',used_product_line.product_id.id)]):
                lot_spl=lot_obj.browse(cr,uid,lot_spl_id)
                #print "lot_spl.hd_qty_virt ", lot_spl.hd_qty_virt
                #print "qty_rest_a_prelev ", qty_rest_a_prelev
                #HDcm 2012_04_16 506_06chem - if lot_spl.hd_qty_virt == 0: continue
                if abs(lot_spl.hd_qty_virt - 0.0) < 0.000000000001: continue#HDcm 2012_04_16 604_02chem 2012_04_16
                if  lot_spl.hd_type == 'fab_stock' and (lot_spl.state != 'conform'): continue
                if  lot_spl.hd_type == 'in' and (lot_spl.state != 'conform'): continue
                if lot_spl.hd_qty_virt >= qty_rest_a_prelev:
    # ATTENTION il faudra tenir compte d'uom différentes
                    res.append({
                        'lot_find_id':lot_spl.id,
                        'qty_prelev':qty_rest_a_prelev,
                        'product_id':used_product_line.product_id.id,
                        'product_uom':used_product_line.product_uom.id,
                        'procure_method':used_product_line.procure_method,
                        'used_product_lines_id':used_product_lines_id,
                        })
                    qty_rest_a_prelev = 0.0
                    break
                else:
                    qty_rest_a_prelev -= lot_spl.hd_qty_virt
                    res.append({
                        'lot_find_id':lot_spl.id,
                        'qty_prelev':lot_spl.hd_qty_virt,
                        'product_id':used_product_line.product_id.id,
                        'product_uom':used_product_line.product_uom.id,
                        'procure_method':used_product_line.procure_method,
                        'used_product_lines_id':used_product_lines_id,
                        })
            #print "qty_rest_a_prelev ", qty_rest_a_prelev
            #hdcm 2013_09_19 v604 if qty_rest_a_prelev > 0:
            if abs(qty_rest_a_prelev -0.0) > 0.000000000001:
                res=[]
        #print "/***_get_lot_spl_mo(mrp_production) - res ",res
        return res    

    # cas des produits non geres par lots
    # calculer pour ce produit la qte restant en stock production 
    # argument : id de la ligne traitée
    # return : qte restante
    def _get_product_remaining_mo(self, cr, uid, ids, used_product_lines_not_managed_id):
        #print "***_get_product_remaining_mo(mrp_production) - ids ",ids, " - used_product_lines_not_managed_id ",used_product_lines_not_managed_id
        used_product_lines_not_managed_obj=self.pool.get('mrp.production.product.line.used').browse(cr,uid,used_product_lines_not_managed_id)
        for production in self.browse(cr,uid,ids):
            # rechercher les picking 'internal' et 'done' correspondant a cette production
            # pour les produits entrés en zone production ou sortis de zone production
            list_picking_in_out = self.pool.get('stock.picking').search(cr, uid, [('hd_production_id_spl', '=', production.id),
                                                                           ('type','=','internal'),
                                                                           ('state','=','done')])
            #print "list_picking_in_out ",list_picking_in_out
            list_move_in = []
            list_move_out = []
            for picking_in in list_picking_in_out:
                #retrouver les moves in 'done' correspondant a ces picking et ce produit
                list_move_in.extend(self.pool.get('stock.move').search(cr, uid, [('picking_id', '=', picking_in),
                                                                           ('product_id','=',used_product_lines_not_managed_obj.product_id.id),
                                                                           ('state','=','done'),
                                                                           ('location_dest_id','=',production.product_id.product_tmpl_id.property_stock_production.id)]))
                #retrouver les moves out 'done' correspondant a ces picking et ce produit
                list_move_out.extend(self.pool.get('stock.move').search(cr, uid, [('picking_id', '=', picking_in),
                                                                           ('product_id','=',used_product_lines_not_managed_obj.product_id.id),
                                                                           ('state','=','done'),
                                                                           ('location_id','=',production.product_id.product_tmpl_id.property_stock_production.id)]))
            #print "list_move_in ",list_move_in
            qty_in = 0.0
            for move_in in self.pool.get('stock.move').browse(cr,uid,list_move_in):
                qty_in += move_in.product_qty
            #print "qty_in ",qty_in
            #print "list_move_out ",list_move_out
            qty_out = 0.0
            for move_out in self.pool.get('stock.move').browse(cr,uid,list_move_out):
                qty_out += move_out.product_qty
            #print "qty_out ",qty_out
            # pour les produits utilises en zone production
            list_picking_used = self.pool.get('stock.picking').search(cr, uid, [('hd_production_id_int', '=', production.id),
                                                                           ('type','=','internal'),
                                                                           ('state','=','done')])
            #print "list_picking_used ",list_picking_used
            list_move_used = []
            for picking_used in list_picking_used:
                #retrouver les moves 'done' correspondant a ces picking et ce produit
                list_move_used.extend(self.pool.get('stock.move').search(cr, uid, [('picking_id', '=', picking_used),
                                                                           ('product_id','=',used_product_lines_not_managed_obj.product_id.id),
                                                                           ('state','=','done')]))
            #print "list_move_used ",list_move_used
            qty_used = 0.0
            for move_used in self.pool.get('stock.move').browse(cr,uid,list_move_used):
                qty_used += move_used.product_qty
            #print "qty_used ",qty_used
        #print "/***_get_product_remaining_mo(mrp_production) - qty_in - qty_used ",qty_in - qty_used
        return qty_in - qty_used - qty_out

    # calculer calculer les quantités restantes en stock MO 
    # return :         
    def _get_remaining_stock_mo(self, cr, uid, ids, context=None):
        col1=35
        col2=15
        for production in self.browse(cr, uid, ids):
            message = 'PRODUCTS IN STOCK FOR %s\n' %(production.name)
            message += string.ljust('',col1+col2,'=')+'\n'
            # traitement des produits gérés par lots    
            passe1=False
            products_inspected = []
            for lot in production.hd_mo_lots_ids_spl:
                if lot.product_id.id not in products_inspected:
                    products_inspected.append(lot.product_id.id)
                    qty_remaining = 0.0
                    for lot2 in production.hd_mo_lots_ids_spl:
                        if lot2.product_id.id == lot.product_id.id:
                            qty_remaining += lot2.hd_qty_virt
                    if not passe1:
                        passe1=True
                        message += 'Products managed by lots\n'
                        message += string.ljust('',col1+col2,'-')+'\n'
                        message += string.ljust('Product',col1)+ string.rjust('Remaining qty',col2)+'\n\n'
                    message += string.ljust(lot.product_id.name,col1,'-')+string.rjust('% 12.6f %s\n'%(
                                                    qty_remaining,
                                                    lot.hd_product_uom_id.name,),col2)
            # traitement des produits non gérés par lots
            passe1=False
            products_inspected = []
            for line in production.hd_product_lines_used_ids:
                if (not line.product_id.hd_track_lot) and (line.product_id.id not in products_inspected):
                    products_inspected.append(line.product_id.id)
                    #print "products_inspected ",products_inspected
                    if not passe1:
                        passe1=True
                        message += '\n\nProducts not managed by lots\n'
                        message += string.ljust('',col1+col2,'-')+'\n'
                        message += string.ljust('Product',col1)+ string.rjust('Remaining qty',col2)+'\n\n'
                    message += string.ljust(line.product_id.name,col1,'-')+string.rjust('% 12.6f %s\n'%(
                                                    self._get_product_remaining_mo(cr,uid,[production.id],line.id),
                                                    line.product_uom.name,),col2)
        return message

    def action_production_end(self, cr, uid, ids, context=None):
        #print "***action_production_end(mrp_production) - ids ",ids
        if context is None: context = {}
        context['MO_production_end'] = ids#HDcm 2012_03_15 v603_02chem
        context['prog'] = 'action_production_end'#HDcm 2012_03_15 v603_02chem
        move_ids = []
        for production in self.browse(cr, uid, ids):
            # verifier que les analyses de cette production sont dans l'etat 'done' ou 'cancel'
            '''hdcm 2013_10_02 voir hd_labo for analysis_mo in production.hd_mo_analysis_ids:
                if not(analysis_mo.state in ('done', 'cancel')):
                    raise osv.except_osv(_('Invalid action !'), _('the analysis of this production order must be in the state \'Done\' or \'Cancelled\''))'''
            for res in production.move_lines:
                move_ids.append(res.id)
            vals= {'state':'confirmed'}
            new_moves = [x.id for x in production.move_created_ids]
            self.pool.get('stock.move').write(cr, uid, new_moves, vals)
            if not production.date_finished:
                self.write(cr, uid, [production.id],
                        {'date_finished': time.strftime('%Y-%m-%d %H:%M:%S')})
            #hdcm2010_04_15
            moves_procur=[]
            #print "production.move_created_ids ", production.move_created_ids
            #print "new_moves ", new_moves
            context['appel_prog'] = True
            self.pool.get('stock.move').force_assign(cr, uid, new_moves, context=context)
            #/HDcm 2012_03_20 v603_02chem
            context['cas'] = 'new_moves'
            self.pool.get('stock.move').action_done(cr, uid, new_moves, context=context)
            # en cas de procur sans SO,traitement du move lié au procur
            #print "moves_procur : ",moves_procur
            if len(moves_procur):
                context['cas'] = 'moves_procur'
                self.pool.get('stock.move').action_done(cr, uid, moves_procur, context=context)
            self._costs_generate(cr, uid, production)
        '''hdcm 2013_10_02 voir hd_labo 
        # recopie de la derniere analyse (en état 'done') de cet MO vers le ou les lots de sortie
        # si le champ 'automatic_analysis_copy' est True
        if production.automatic_analysis_copy:
            # trouver la derniere analyse effectuee avant la fin du MO ayant un etat 'done'
            analysis_mo_ids = [x.id for x in production.hd_mo_analysis_ids]
            while analysis_mo_ids:
                analysis_mo_id = analysis_mo_ids.pop()
                if self.pool.get('labo.analysis').browse(cr, uid, analysis_mo_id).state == 'done':
                    #print "analysis_mo_id ",analysis_mo_id
                    # creer pour chaque lot de sortie une analyse identique
                    self.pool.get('labo.analysis').copy_labo_analysis(cr, uid, analysis_mo_id, [x.id for x in production.hd_mo_lots_ids_fab], fab=True)
                    break'''
        self.write(cr,  uid, ids, {'state': 'production_end'})
        #print "/***action_production_end(mrp_production)"
        return True

    def action_production_done(self, cr, uid, ids, context=None):
        #print "***action_production_done(mrp_production) - ids ",ids, " - context ",context
        if context is None: context = {}
        #hdcm 2013_02_27 verifier s'il reste des matieres en stock production
        for production in self.browse(cr, uid, ids):
            # traitement des produits gérés par lots    
            passe1=False
            products_inspected = []
            for lot in production.hd_mo_lots_ids_spl:
                if lot.product_id.id not in products_inspected:
                    products_inspected.append(lot.product_id.id)
                    qty_remaining = 0.0
                    for lot2 in production.hd_mo_lots_ids_spl:
                        if lot2.product_id.id == lot.product_id.id:
                            qty_remaining += lot2.hd_qty_virt
                    if not passe1:
                        passe1=True
                if abs(qty_remaining - 0.0) > 0.000000000001:
                    raise osv.except_osv(_('ATTENTION !'), _('Products are still stored in the production zone, you must return them to the stock before finishing the production order.'))
            # traitement des produits non gérés par lots
            passe1=False
            products_inspected = []
            for line in production.hd_product_lines_used_ids:
                qty_remaining = 0.0
                if (not line.product_id.hd_track_lot) and (line.product_id.id not in products_inspected):
                    products_inspected.append(line.product_id.id)
                    if not passe1:
                        passe1=True
                    qty_remaining = self._get_product_remaining_mo(cr,uid,[production.id],line.id)
            if abs(qty_remaining - 0.0) > 0.000000000001:
                raise osv.except_osv(_('ATTENTION !'), _('Products are still stored in the production zone, you must return them to the stock before finishing the production order.'))
        self.write(cr, uid, ids, {'state':'done'})
        #print "***action_production_done(mrp_production) - ids ",ids, " - context ",context
        return True    
    
#/hdcm        
    # retour des reliquats des lots splits d'une fab
    # Principe : 
    #    -    si le lot à retourner a le même état que le lot d'origine, faire une opération de transfert
    #         sinon il faudra faire un stock_move_lot manuel
    def return_to_stock(self, cr, uid, ids,  context={}):
        #print "***return_to_stock(mrp_production) - ids ",ids, " - context ",context
        for production in self.browse(cr, uid, ids):
            # traitement des produits geres
            self.pool.get('stock.production.lot').return_to_stock_one_lot(cr, uid, [x.id for x in production.hd_mo_lots_ids_spl], context=context)
            # traitement des produits non geres
            products_inspected = []
            picking_id3=False
            #HDcm 2012_04_15 604_02chem - ne rien faire si toutes les lignes sont vides
            toutes_les_lignes_vides_produits_ng = True
            for line in production.hd_product_lines_used_ids:
                if (not line.product_id.hd_track_lot) and \
                    not (abs(self._get_product_remaining_mo(cr,uid,[production.id],line.id) - 0.00) < 0.000000000001):
                    toutes_les_lignes_vides_produits_ng = False
                    break
            if toutes_les_lignes_vides_produits_ng: 
                #print "/***return_to_stock(mrp_production)"
                return True
            for line in production.hd_product_lines_used_ids:
                if (not line.product_id.hd_track_lot) and (line.product_id.id not in products_inspected):
                    #print "product ", line.product_id.name, " - qty ", self._get_product_remaining_mo(cr,uid,[production.id],line.id)
                    products_inspected.append(line.product_id.id)
                    # calculer la qte a retourner
                    # retrouver la location du 1er move du 1er picking in concernant ce produit
                    # rechercher les picking 'internal' et 'done' correspondant a cette production
                    # pour les produits entrés en zone production ou sorti de la zone production
                    list_picking_in_out = self.pool.get('stock.picking').search(cr, uid, [('hd_production_id_spl', '=', production.id),
                                                                                  ('type','=','internal'),
                                                                                  ('state','=','done')])
                    list_move_in = []
                    for picking_in in list_picking_in_out:
                        #retrouver les moves 'done' correspondant a ces picking et ce produit
                        list_move_in.extend(self.pool.get('stock.move').search(cr, uid, [('picking_id', '=', picking_in),
                                                                                ('product_id','=',line.product_id.id),
                                                                                ('state','=','done'),
                                                                                ('location_dest_id','=',production.product_id.product_tmpl_id.property_stock_production.id)]))
                    # creer le picking
                    if not picking_id3:
                        # création d'un picking contenant tous ces moves
                        picking_id3 = self.pool.get('stock.picking').create(cr, uid, {
                            'origin': (production.origin or '') +':'+production.name,
                            'type': 'internal',
                            'move_type': 'one',
                            'state': 'auto',
                            'auto_picking': True,
                            'hd_production_id_spl': production.id,
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                            })
                    # creer le move (en etat done) retournant ce produit en stock
                    data = {
                            'name':'PROD:'+production.name,
                            'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
                            'product_id': line.product_id.id,
                            'product_qty': self._get_product_remaining_mo(cr,uid,[production.id],line.id),
                            'product_uom': line.product_uom.id,
                            'location_id': production.product_id.product_tmpl_id.property_stock_production.id,
                            'location_dest_id': self.pool.get('stock.move').browse(cr, uid, list_move_in[0]).location_id.id,
                            'picking_id':picking_id3,
                             #HDcm 2012_04_15 v506_06chem'state': 'done',
                            'state': 'draft', #HDcm 2012_04_15 v604_02chem
                            'company_id': production.company_id.id,#HDcm 2012_03_08 v603_01chem
                            'move_history_ids': [], #HDcm 2012_04_15 v604_02chem
                            'move_history_ids2': [], #HDcm 2012_04_15 v604_02chem
                    }
                    self.pool.get('stock.move').create(cr, uid, data)
            if picking_id3:
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_validate(uid, 'stock.picking', picking_id3, 'button_confirm', cr)
                wf_service.trg_validate(uid, 'stock.picking', picking_id3, 'button_done', cr)
        #print "/***return_to_stock(mrp_production)"
        return True

    def on_change_hd_nb_production_lot(self, cr, uid, ids, hd_nb_production_lot):
        #print "***on_change_hd_nb_production_lot(mrp_production) - ids ",ids, " - hd_nb_production_lot ", hd_nb_production_lot
        result = {}
        warning = {}
        if hd_nb_production_lot < 1:
            result['hd_nb_production_lot'] = self.read(cr, uid, ids, ['hd_nb_production_lot'])[0]['hd_nb_production_lot']
            warning={'title':_('UserError'), 'message':_('the number of lot must be at least equal to 1')}
        #print "/***on_change_hd_nb_production_lot(mrp_production) - result ",result
        return {'value':result, 'warning':warning}

    def product_id_change(self, cr, uid, ids, product_id, context=None):
        """ Finds UoM of changed product.
        @param product_id: Id of changed product.
        @return: Dictionary of values.
        """
        #print "***product_id_change(mrp_production) - ids ",ids, " - product_id ", product_id, " - context ", context
        res = super(mrp_production, self).product_id_change(cr, uid, ids, product_id, context=context)
        product_saisi = self.pool.get('product.product').browse(cr, uid, product_id)
        #res['domain']['product_uom'] = [('id','in',self.pool.get('product.uom').search(cr, uid, [('category_id', '=', product_saisi.uom_id.category_id.id)]))]
        res['domain'] = {'product_uom': [('id','in',self.pool.get('product.uom').search(cr, uid, [('category_id', '=', product_saisi.uom_id.category_id.id)]))]}
        
        #print "/***product_id_change(mrp_production) - res ",res
        return res


#!!! mrp_production()

# Ajout d'une nouvelle classe correspondant aux lignes de production utilisées dans le MO 
class mrp_production_product_line_used(osv.osv):
    _name = 'mrp.production.product.line.used'
    _description = 'Production used products'
    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'product_id': fields.many2one('product.product', 'Product', required=True),
        'qty_bom': fields.float('Bom Qty', required=True, digits_compute=dp.get_precision('Qty Module Process')),
        'qty_lab': fields.float('Lab Qty', required=True, digits_compute=dp.get_precision('Qty Module Process')),
        'qty_pick': fields.float('Picking Qty', required=True, digits_compute=dp.get_precision('Qty Module Process')),
        'qty_prod': fields.float('Production Qty', required=True, digits_compute=dp.get_precision('Qty Module Process')),
        'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
        'production_id_waiting': fields.many2one('mrp.production', 'Production Order'),
        'production_id_procur': fields.many2one('mrp.production', 'Production Order'),
        'production_id_used': fields.many2one('mrp.production', 'Production Order'),
        'move_id_spl': fields.integer('Move split'),
        'move_id_int': fields.integer('Move int'),
        'procure_method': fields.selection([('make_to_stock','Make to Stock'),('make_to_order','Make to Order')], 'Procure Method'),
        'supply_method': fields.selection([('produce','Produce'),('buy','Buy')], 'Supply method'),
        'sel':fields.boolean('Select'),
        'note': fields.text('Notes', states={'procur_run':[('readonly',True)], 'procur_done':[('readonly',True)], 'waiting_product':[('readonly',True)], 'used':[('readonly',True)], 'cancel':[('readonly',True)]}),
        'state': fields.selection([
                        ('draft','Draft'),
                        ('procur_run','Procurement running'),
                        ('procur_done','Procurement done'),
                        ('waiting_product','waiting product'),
                        ('used','Used'),
                        ('cancel','Canceled'),
                    ],'State', readonly=True),
        'lot_org': fields.many2one('stock.production.lot', 'Lot of origin'),
        'type_line': fields.selection([
                        ('standard','Standard'),    
                        ('reprocessing','Reprocessing'),
                    ],'Type Line', required=True, help="Reprocessing : The proposed lots are in the state \'to reprocess\'"),
    }
    
    _defaults = {
        'qty_bom':  lambda *a: 0.0,
        'qty_lab':  lambda *a: 0.0,
        'qty_pick':  lambda *a: 0.0,
        'qty_prod':  lambda *a: 0.0,
        'sel': lambda *a: 1,
        'procure_method': lambda *a: 'make_to_stock',
        'supply_method': lambda *a: 'buy',
        'state': lambda *a: 'draft',
        'type_line': lambda *a: 'standard',
    }

    # autoriser unlink MO uniquement pour l'etat draft
    def unlink(self, cr, uid, ids, context=None):
        #print "***unlink(mrp_production_product_line_used) - ids ",ids
        mrp_production_product_line_used = self.read(cr, uid, ids, ['state'])
        unlink_ids = []
        for s in mrp_production_product_line_used:
            if s['state'] in ['draft']:
                unlink_ids.append(s['id'])
            else:
                raise osv.except_osv(_('Invalid action !'), '%s %s %s' % (_('Cannot delete Production product line(s) which are in'), s['state'], _('state !')))#hdcm 2012_11_27
        #print "/***unlink(mrp_production_product_line_used)"
        return osv.osv.unlink(self, cr, uid, unlink_ids, context=context)

    def button_cancel(self, cr, uid, ids, context=None):
        #print "***button_cancel(mrp_production_product_line_used) - ids ",ids
        for line in self.browse(cr, uid, ids):
            # cas production_id_used - autoriser cancel pour l'etat 'waiting_product'
            if line.production_id_used:
                #print "cas production_id_used"
                if line.state in ('waiting_product','cancel'):
                    self.write(cr, uid, [line.id], {'state': 'cancel'})
                    # retrouver la ligne production_id_procur correspondante (si elle existe) et faire cancel
                    if self.search(cr, uid, [('move_id_spl', '=', line.move_id_spl), 
                                        ('production_id_procur','=',line.production_id_used.id)]):
                        line_production_id_procur=self.search(cr, uid, [('move_id_spl', '=', line.move_id_spl), 
                                        ('production_id_procur','=',line.production_id_used.id)])[0]
                        self.write(cr, uid, [line_production_id_procur], {'state': 'cancel'})
                        # retrouver le move puis le picking correspondant et le mettre a 'cancel' si etat 'draft'
                        picking_obj = self.pool.get('stock.move').browse(cr, uid, line.move_id_spl).picking_id
                        if picking_obj.state in ('draft','assigned'):
                            self.pool.get('stock.picking').action_cancel(cr, uid,[picking_obj.id])
                else:
                    raise osv.except_osv(_('Invalid action !'), '%s %s %s' % (_('Cannot cancel product line(s) which are in'), line.state, _('state !')))#hdcm 2012_11_28
            # cas production_id_waiting - autoriser cancel pour l'etat 'draft'
            if line.production_id_waiting:
                #print "cas production_id_waiting"
                if line.state in ('draft','cancel'):
                    self.write(cr, uid, [line.id], {'state': 'cancel'})
                else:
                    raise osv.except_osv(_('Invalid action !'), '%s %s %s' % (_('Cannot cancel product line(s) which are in'), line.state, _('state !')))#hdcm 2012_11_28
        #print "/***button_cancel(mrp_production_product_line_used)"
        return True

    def product_id_change_mo_normal(self, cr, uid, ids, product, uom, type_line, mo_product_id):
        #print " ***product_id_change_mo_normal(mrp_production_product_line_used) - ids ", ids, " - product ", product, " - uom ", uom, " - type_line ", type_line, " - mo_product_id ", mo_product_id
        #hdcm 2013_09_05 v604 res = {'value': {},'warning': {}}
        res = {'value': {},'warning': {},'domain': {}}
        if (type_line == 'reprocessing'):
            if (not product) or (product != mo_product_id):
                if mo_product_id:
                    return {'value': {'product_id': mo_product_id},
                            'warning': {'title':_('Information'), 'message':_('You can not edit this field')}}
                else:
                    return {'value': {'name':'', 'product_uom':False, 'lot_org':False}}
            else:
                product_saisi = self.pool.get('product.product').browse(cr, uid, product)
                res['value'] = {'name': product_saisi.name,
                                'product_uom': product_saisi.uom_id.id, 
                                'procure_method': 'make_to_stock', 
                                'supply_method': product_saisi.supply_method}
        elif (type_line == 'standard'):
            if not product:
                return {'value': {'name':'', 'product_uom':False, 'lot_org':False}}
            product_saisi = self.pool.get('product.product').browse(cr, uid, product)
            res['value'] = {'name': product_saisi.name,
                            'product_uom': product_saisi.uom_id.id, 
                            'procure_method': product_saisi.procure_method, 
                            'supply_method': product_saisi.supply_method}
        #hdcm 2013_09_05 v7.0
        product_saisi = self.pool.get('product.product').browse(cr, uid, product)
        res['domain'] = {'product_uom': [('id','in',self.pool.get('product.uom').search(cr, uid, [('category_id', '=', product_saisi.uom_id.category_id.id)]))]}
        #/hdcm 2013_09_05 v7.0
        #print "/***product_id_change_mo_normal(mrp_production_product_line_used) - res ", res
        return res    

    def product_uom_change_mo_normal(self, cr, uid, ids, product, uom, type_line):
       #print " ***product_uom_change_mo_normal(mrp_production_product_line_used) - ids ", ids, " - product ", product, " - uom ", uom, " - type_line ", type_line
        res = {'value': {},'warning': {}}
        if (type_line == 'reprocessing'):
            product_saisi = self.pool.get('product.product').browse(cr, uid, product)
            if (not uom) or (uom != product_saisi.uom_id.id):
                res['value'] = {'product_uom': product_saisi.uom_id.id}
                res['warning'] = {'title':_('Information'), 'message':_('You can not edit this field')}
       #print "/***product_uom_change_mo_normal(mrp_production_product_line_used) - res ", res
        return res    

    def product_qty_change_mo_normal(self, cr, uid, ids, product, lot_org, qty_prod, qty_pick, type_line):
       #print " ***product_qty_change_mo_normal(mrp_production_product_line_used) - ids ", ids, " - product ", product, " - qty_prod ", qty_prod, " - qty_pick ", qty_pick, " - type_line ", type_line
        res = {'value': {},'warning': {}}
        if (type_line == 'reprocessing') and lot_org:
            product_saisi = self.pool.get('product.product').browse(cr, uid, product)
            lot = self.pool.get('stock.production.lot').browse(cr, uid, lot_org)
            if (qty_prod != lot.hd_qty_virt) or (qty_pick != lot.hd_qty_virt):
                res['value'] = {'qty_prod': lot.hd_qty_virt, 'qty_pick': lot.hd_qty_virt}
                res['warning'] = {'title':_('Information'), 'message':_('You can not edit this field')}
       #print "/***product_qty_change_mo_normal(mrp_production_product_line_used) - res ", res
        return res    


    def lot_org_change_mo_normal(self, cr, uid, ids, lot_org, qty, uom, product, location, type_line):#HDcm 2012_05_31 604_03chem
       #print " ***lot_org_change_mo_normal(mrp_production_product_line_used) - ids ", ids, " - lot_org ", lot_org, " - qty ", qty, " - uom ", uom, \
       #                                              " - product ", product, " - location ", location, " - type_line ", type_line
        res = {'value': {},'warning': {}}
        prod_line_used_obj = self.pool.get('mrp.production.product.line.used')
        if type_line == 'reprocessing':
            if lot_org:
                lot_saisi = self.pool.get('stock.production.lot').browse(cr, uid, lot_org)
                # verifier que ce lot (avec l'etat 'to reprocess') n'a pas deja ete dans ce MO ou dans un autre MO
                if len(prod_line_used_obj.search(cr, uid, [('lot_org', '=',lot_org),('type_line', '=', 'reprocessing'),('state', '!=', 'cancel')])):
                    res['value'] = {'lot_org': False, 'qty_prod': 0.0, 'qty_pick': 0.0}
                    res['warning'] = {'title':_('Information'), 'message':_('This lot \'To Reprocess\' has already been selected in this manufacturing order or in a another')}
                else:
                    lot = self.pool.get('stock.production.lot').browse(cr, uid, lot_org)
                    res['value'] = {'name':lot_saisi.product_id.name,'qty_prod': lot.hd_qty_virt, 'qty_pick': lot.hd_qty_virt}
            elif not lot_org:
                res['value'] = {'qty_prod': 0.0, 'qty_pick': 0.0}
        elif type_line =='standard':
            if not product:
                return {'value': {'product_uom':False, 'lot_org':False}}
            context = {'cas_mo_normal_drafting_lines': True, 'product_id': product, 'location_ids': [location]}
            context.update(CAS_MO_NORMAL)
            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, [lot_org], qty, uom, context=context)
            if verif_lots['infos'] != '': 
                res['value']['lot_org'] = False
                res['warning']={'title':_('ATTENTION'), 'message': verif_lots['infos']}
       #print "/***lot_org_change_mo_normal(mrp_production_product_line_used) - res ", res
        return res
        
    def qty_lab_change(self, cr, uid, ids, qty_lab):
        res = {'value': {'qty_prod':qty_lab or 0.0}}
        return res

    #hdcm 2013_02_21
    def type_line_change(self, cr, uid, ids, type_line, mo_product_id, context=None):#HDcm 2012_05_31 604_03chem
       #print " ***type_line_change(mrp_production_product_line_used) - ids ", ids, " - type_line ", type_line, " - mo_product_id ", mo_product_id, " - context ", context
        if context is None: context = {}
        res = {'value': {},'warning': {}}
        if type_line == 'standard':
            res['value'] = {'product_id': False, 'product_uom': False,'lot_org': False,'procure_method': False,'supply_method': False,
                            'qty_bom': 0.0,'qty_lab': 0.0,'qty_prod': 0.0,'qty_pick': 0.0}
        elif type_line == 'reprocessing':
            if not mo_product_id:
                res['value'] = {'product_id': False, 'product_uom': False,'lot_org': False,'procure_method': False,'supply_method': False,
                            'qty_bom': 0.0,'qty_lab': 0.0,'qty_prod': 0.0,'qty_pick': 0.0}
                res['warning'] = {'title':_('Information'), 'message':_('Enter the product to be manufactured !')}
            else:
                product = self.pool.get('product.product').browse(cr, uid, mo_product_id)
                res['value'] = {'product_id': mo_product_id, 'product_uom': product.uom_id.id,'lot_org': False,'procure_method': 'make_to_stock',
                                'supply_method': product.supply_method,
                            'qty_bom': 0.0,'qty_lab': 0.0,'qty_prod': 0.0,'qty_pick': 0.0}
       #print "/***type_line_change(mrp_production_product_line_used) - res ", res
        return res

mrp_production_product_line_used()

#hdcm2012_09_10 604_04 dans l'onglet 'Bom products' d'un MO, passer les quantités sur 6 décimales
class mrp_production_product_line(osv.osv):
    _name = 'mrp.production.product.line'
    _inherit = 'mrp.production.product.line'
    _columns = {
        'product_qty': fields.float('Product Qty', digits_compute=dp.get_precision('Qty Module Process'), required=True),
        'product_uos_qty': fields.float('Product UOS Qty', digits_compute=dp.get_precision('Qty Module Process')),
    }
#!!! mrp_production_product_line()

class mrp_bom(osv.osv):
    _name = 'mrp.bom'
    _inherit = 'mrp.bom'

    _columns = {
        'product_qty': fields.float('Product Qty', digits_compute=dp.get_precision('Qty Module Process'), required=True),
            }

#!!! mrp_bom()

class remaining_stock_mo(osv.osv_memory):
    _name='remaining.stock.mo'

    def _get_info_stock_mo(self,cr, uid, ctx):
        if 'active_id' in ctx:
            return self.pool.get('mrp.production')._get_remaining_stock_mo(cr,uid,[ctx['active_id']])
        return False

    _columns = {
        'note': fields.text('Information for stock production in MO'),
            }

    _defaults = {
        'note': _get_info_stock_mo
        }

remaining_stock_mo()