# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _

class fill_reprocessing_lot(osv.osv_memory):
    _name = "fill.reprocessing.lot"
    _description = "Select lots to reprocess"

    def _init_product_id(self, cr, uid, context=None):
        #print " ***_init_product_id(fill_reprocessing_lot) - context ", context
        if context is None: context = {}
        res = False
        if context.get('active_id', False):
            prod = self.pool.get('mrp.production').browse(cr, uid, context.get('active_id'))
            if prod.product_id : res = prod.product_id.id
        return res

    def _init_product_uom(self, cr, uid, context=None):
        #print " ***_init_product_uom(fill_reprocessing_lot) - context ", context
        if context is None: context = {}
        res = False
        if context.get('active_id', False):
            prod = self.pool.get('mrp.production').browse(cr, uid, context.get('active_id'))
            if prod.product_id : res = prod.product_id.uom_id.id
        return res

    def _init_info(self, cr, uid, context=None):
        #print " ***_init_info(fill_reprocessing_lot) - context ", context
        if context is None: context = {}
        res = False
        if context.get('active_id', False):
            prod = self.pool.get('mrp.production').browse(cr, uid, context.get('active_id'))
            if prod.product_id : res = "%s : %s / %s : %s" % (_('Product'), prod.product_id.name, _('Uom'), prod.product_id.uom_id.name)
        return res

    def _init_location_id(self, cr, uid, context=None):
        #print " ***_init_location_id(fill_reprocessing_lot) - context ", context
        if context is None: context = {}
        res = False
        if context.get('active_id', False):
            prod = self.pool.get('mrp.production').browse(cr, uid, context.get('active_id'))
            if prod.location_src_id: res = prod.location_src_id.id
        return res

    _columns = {
        'product_id': fields.many2one('product.product', 'Product'),
        'product_uom': fields.many2one('product.uom', 'Product UOM'),
        'info': fields.char('Info', size=128, readonly=True),
        'location_id': fields.many2one('stock.location', 'Location', required=True),
        #'lot_reprocessing_ids': fields.many2many('stock.production.lot', 'fill_reprocessing_lots_rel', 'fill_reprocessing_id','lot_id','Lots to reprocess'
                                                 #, domain="[('state','=','to_reprocess')]"),
        'lot_reprocessing_ids': fields.many2many('stock.production.lot', 'fill_reprocessing_lots_rel', 'fill_reprocessing_id','lot_id','Lots to reprocess'),
    }

    _defaults = {
        'product_id': _init_product_id,
        'product_uom': _init_product_uom,
        'info': _init_info,
        'location_id': _init_location_id,
    }

    def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None: context = {}
        super(fill_reprocessing_lot, self).view_init(cr, uid, fields_list, context=context)
        if len(context.get('active_ids',[])) > 1:
            raise osv.except_osv(_('Error!'), _('You cannot perform this operation on more than one manufacturing order.'))
        if context.get('active_id', False):
            prod = self.pool.get('mrp.production').browse(cr, uid, context.get('active_id', False))
            if prod.state not in ('draft', 'in_production'):
                raise osv.except_osv(_('Warning!'), _('The manufacturing order must be in the state \'draft\' or \'in production\'.'))
        return True

    def action_fill_reprocessing_lot(self, cr, uid, ids, context=None):
        """ To Import stock inventory according to products available in the selected locations.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: the ID or list of IDs if we want more than one
        @param context: A standard dictionary
        @return:
        """
       #print " ***action_fill_reprocessing_lot(fill_reprocessing_lot) - ids ", ids, " - context ", context
        if context is None: context = {}
        # creer les 'product lines xaiting'
        if context.get('active_id', False):
            prod_line_used_obj = self.pool.get('mrp.production.product.line.used')
            prod = self.pool.get('mrp.production').browse(cr, uid, context.get('active_id'))
            # verifier a pour lot qu'il n'est pas deja utilise dans ce MO ou un autre
            lots_refused = []
            for lot in self.browse(cr, uid, ids[0]).lot_reprocessing_ids:
                if len(prod_line_used_obj.search(cr, uid, [('lot_org', '=',lot.id),('type_line', '=', 'reprocessing'),('state', '!=', 'cancel')])):
                    lots_refused.append(lot.id)
                else: # creeer la ligne
                    drafting_line = {'production_id_waiting': prod.id,
                                   'name': lot.product_id.name,
                                   'type_line': 'reprocessing',
                                   'lot_org': lot.id,
                                   'product_id': lot.product_id.id,
                                   'product_uom': lot.product_id.uom_id.id,
                                   'procure_method': 'make_to_stock', 
                                   'supply_method': lot.product_id.supply_method,
                                   'qty_prod': lot.hd_qty_virt, 
                                   'qty_pick': lot.hd_qty_virt,}
                    prod_line_used_obj.create(cr, uid, drafting_line)
       #print "lots_refused ", lots_refused
        return {'type': 'ir.actions.act_window_close'}
fill_reprocessing_lot()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
