# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

import netsvc
import time

from osv import osv,fields
from tools.translate import _
import decimal_precision as dp

class production_end_memory(osv.osv_memory):
    _name = "production.end.memory"

    _columns = {
        'name': fields.char('', size=64, readonly=True),    
        'lot_qty' : fields.float('', digits_compute=dp.get_precision('Qty Module Chem'), required=True),
        'wizard_id' : fields.many2one('production.end', string="Wizard"),
    }

production_end_memory()

class production_end(osv.osv_memory):
    _name = 'production.end'
    _description = 'Production end'

    _columns = {
        'info': fields.char('Infos MO', size=128, readonly=True),    
        'lot_qty_lines' : fields.one2many('production.end.memory', 'wizard_id', 'Lots qty'),
    }

    def default_get(self, cr, uid, fields, context=None):
        """
         To get default values for the object.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param fields: List of fields for which we want default values
         @param context: A standard dictionary
         @return: A dictionary with default values for all field in ``fields``
        """
        #print " ***default_get(production_end) - fields ", fields, " - context ", context
        if context is None: context = {}
        res = super(production_end, self).default_get(cr, uid, fields, context=context)
        record_id = context and context.get('active_id', False) or False
        prod_obj = self.pool.get('mrp.production')
        prod = prod_obj.browse(cr, uid, record_id, context=context)
        # vérifier que les conditions sont remplies pour exécuter ce wizard
        i = 0
        for product_line_used in prod.hd_product_lines_used_ids:
            i += 1
            if product_line_used.state not in ('used','cancel'):
                raise osv.except_osv(_('UserError'), _('the In process product lines must be in state \'used\''))
        if i==0:
            raise osv.except_osv(_('UserError'), _('the zone \'In process product lines\' must have at least a line'))
        i=0
        result1 = []
        for move_created_obj in prod.move_created_ids:
            #v604 res.update({'qty_lot%s' % (i+1): move_created_obj.product_qty})  
            result1.append({'name': 'Qty lot%s' % (i+1), 'lot_qty': move_created_obj.product_qty})
            i += 1
        if 'lot_qty_lines' in fields:
            res.update({'lot_qty_lines': result1})
        #print "/***default_get(production_end) - res ", res
        return res

    def save_qty(self, cr, uid, ids, context=None):
        """ Splits moves in quantity given in the wizard.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of ids selected 
         @param context: A standard dictionary 
         @return: A dictionary which of fields with values. 
        """ 
        #print " ***save_qty(production_end) - ids ", ids, " - context ", context
        if context is None: context = {}
        record_id = context and context.get('active_id', False) or False
        prod_obj = self.pool.get('mrp.production')
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_04_14 604_02chem
        prod = prod_obj.browse(cr, uid, record_id, context=context)
        data = self.read(cr, uid, ids[0], context=context)
        # verifier que le nbre de ligne est correct
        if len(data['lot_qty_lines']) != prod.hd_nb_production_lot:
            raise osv.except_osv(_('UserError'), _('You can not add or delete lines'))
        # vérifier que les qty des lots sont positives ou nulles et qu'une ligne n'a pas ete creee manuellement
        list_qty = []
        for line_id in data['lot_qty_lines']:
            line_qty_obj = self.pool.get('production.end.memory')
            line_qty = line_qty_obj.browse(cr, uid, line_id, context=context)
            if line_qty.lot_qty < 0:
                raise osv.except_osv(_('UserError'), _('the lots quantity must be positive or null'))
            if not line_qty.name:
                raise osv.except_osv(_('UserError'), _('You manually created lines, please re-run the \'Production end\''))
            list_qty.append(line_qty.lot_qty)
        # recalcul de la qty totale et réécriture dans mrp.production
        recalcul_tot_qty=0
        i=0
        for move_created_obj in prod.move_created_ids:
            recalcul_tot_qty += list_qty[i]
            i += 1
        
        prod_obj.write(cr,uid, [record_id], {'product_qty':recalcul_tot_qty})
        #HDcm 2012_04_14 604_02chem - modifier les hd_qty_org pour les lots de sortie
        i=0
        for lot_fab in prod.hd_mo_lots_ids_fab:
            lot_obj.write(cr, uid, [lot_fab.id], {'hd_qty_org': list_qty[i]})
            i += 1
        # réactualiser les moves
        move_obj = self.pool.get('stock.move')
        i=0
        for move_created_obj in prod.move_created_ids:
            move_obj.write(cr, uid, [move_created_obj.id],{'product_qty': list_qty[i],})
            i += 1
        # lancer la fonction action_production_end
        prod_obj.action_production_end(cr, uid, [record_id])
        # si cette production est liée à un procurement, modifier la qty du procurement
        
        # si cette production est liée à une ligne de SO, modifier la qty de cette ligne de SO 
        
        # actionner le workflow
        wf_service = netsvc.LocalService("workflow")
        wf_service.trg_validate(uid, 'mrp.production', record_id, 'button_mo_end', cr)
        #print "/***save_qty(production_end)"
        return {'type': 'ir.actions.act_window_close'}

production_end()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

