# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 


import time
import netsvc
from osv import fields, osv, orm
#hdcm 20130704 import ir
from mx import DateTime
from tools import config
from tools.translate import _

CAS_SO_NORMAL = {'types': ('in', 'fab_stock', 'split','sale_fab'), 'states': ('conform','alert_conform'), 'location_under_control': 'not_allowable'}

class sale_order(osv.osv):
    _name = 'sale.order'
    _inherit = 'sale.order'
    
    def _prepare_order_line_move(self, cr, uid, order, line, picking_id, date_planned, context=None):
        location_id = order.shop_id.warehouse_id.lot_stock_id.id
        output_id = order.shop_id.warehouse_id.lot_output_id.id
        #hdcm 2013_07_25
        hd_lot_org = 0
        if line.hd_lot_org.id:hd_lot_org = line.hd_lot_org.id
        #/hdcm 2013_07_25
        return {
            'name': line.name,
            'picking_id': picking_id,
            'product_id': line.product_id.id,
            'date': date_planned,
            'date_expected': date_planned,
            'product_qty': line.product_uom_qty,
            'product_uom': line.product_uom.id,
            'product_uos_qty': (line.product_uos and line.product_uos_qty) or line.product_uom_qty,
            'product_uos': (line.product_uos and line.product_uos.id)\
                    or line.product_uom.id,
            'product_packaging': line.product_packaging.id,
            'partner_id': line.address_allotment_id.id or order.partner_shipping_id.id,
            'location_id': location_id,
            'location_dest_id': output_id,
            'sale_line_id': line.id,
            'tracking_id': False,
            'state': 'draft',
            #'state': 'waiting',
            'company_id': order.company_id.id,
            'price_unit': line.product_id.standard_price or 0.0,
            #hdcm 2013_07_25    add source lot
            'hd_lot_org':hd_lot_org,
            #/hdcm 2013_07_25
        }

    def action_view_delivery(self, cr, uid, ids, context=None):
        '''
        This function returns an action that display existing delivery orders of given sales order ids. It can either be a in a list or in a form view, if there is only one delivery order to show.
        '''
        #print " ***action_view_delivery(sale_order) _ ids %s - context %s"%(ids, context)
        mod_obj = self.pool.get('ir.model.data')
        act_obj = self.pool.get('ir.actions.act_window')

        result = mod_obj.get_object_reference(cr, uid, 'stock', 'action_picking_tree')
        id = result and result[1] or False
        result = act_obj.read(cr, uid, [id], context=context)[0]
        #compute the number of delivery orders to display
        pick_ids = []
        for so in self.browse(cr, uid, ids, context=context):
            pick_ids += [picking.id for picking in so.picking_ids]
        #choose the view_mode accordingly
        if len(pick_ids) > 1:
            result['domain'] = "[('id','in',["+','.join(map(str, pick_ids))+"])]"
        else:
            #hdcm 2014_01_03 res = mod_obj.get_object_reference(cr, uid, 'stock', 'view_picking_out_form')
            res = mod_obj.get_object_reference(cr, uid, 'hd_process', 'view_delivery_orders_form')
            result['views'] = [(res and res[1] or False, 'form')]
            result['res_id'] = pick_ids and pick_ids[0] or False
        #print "/***action_view_delivery(sale_order) _ result %s"%(result)
        return result

sale_order()

class sale_order_line(osv.osv):
    _name = 'sale.order.line'
    _inherit = 'sale.order.line'

    #hdcm 2010_01_18
    def _qty_already_delivered(self, cr, uid, ids, field_name, arg, context):
        #print "***_qty_already_delivered(sale.order.line) - ids ",ids
        res = {}
        for line in self.browse(cr, uid, ids):
            res[line.id] = 0
            # rechercher les moves correspondant a cette ligne et etat = 'done' 
            # et location_dest_id = property_stock_customer de ce customer
            qty_already_delivered = 0
            moves = self.pool.get('stock.move').search(cr, uid, [('sale_line_id','=',line.id),
                                                                         ('state','=', 'done'),
                                   ('location_dest_id','=', line.order_id.partner_id.property_stock_customer.id) ])
            for move_obj in self.pool.get('stock.move').read(cr, uid, moves, ['product_qty']):
                qty_already_delivered += move_obj['product_qty']
            res[line.id] = qty_already_delivered
        #print "/***_qty_already_delivered(sale.order.line) - res ",res
        return res

    def _get_qty_already_delivered(self, cr, uid, ids, context={}):
        #print "***_get_qty_already_delivered(sale.order.line) - ids ",ids," - context ",context
        result = {}
        for move in self.pool.get('stock.move').browse(cr, uid, ids, context=context):
            result[move.sale_line_id.id] = True
        #print "result ",result
        #print "/***_get_qty_already_delivered(sale.order.line) - result.keys() ",result.keys()
        return result.keys()

    _columns = {
        'hd_lot_org': fields.many2one('stock.production.lot', 'Lot of origin'),
        'hd_qty_already_delivered': fields.function(_qty_already_delivered, method=True, string='Qty already delivered',
                            store={'stock.move': (_get_qty_already_delivered, None, 10),}),
        'hd_track_lot': fields.related('product_id', 'hd_track_lot', type='boolean', string='Manage by lot', store=True)#hdcm 2013_02_05
    }

    def product_id_change(self, cr, uid, ids, pricelist, product, qty=0,
            uom=False, qty_uos=0, uos=False, name='', partner_id=False,
            lang=False, update_tax=True, date_order=False, packaging=False, fiscal_position=False, flag=False, lot_org_id =False, context=None):
        #print " ***product_id_change(sale_order_line) ", ids," - pricelist ",pricelist," - product ",product," - qty ",qty," - lot_org_id ",lot_org_id," - context ",context
        res=super(sale_order_line,self).product_id_change(cr, uid, ids, pricelist, product, qty, uom, qty_uos, uos, name, partner_id,lang, update_tax, date_order, packaging, fiscal_position, flag, context=context)
        #print "1er res ", res
        if context is None: context = {}#hdcm 2013_07_26
        if not product:
            res['value']['hd_lot_org'] = False
            res['value']['product_uom_qty'] = 0.0
            res['value']['product_uom'] = False
            res['value']['product_uos_qty'] = 0.0
            res['value']['product_uos'] = False
            res['value']['name'] = False
            res['value']['price_unit'] = 0.0
            res['value']['discount'] = 0.0
           #print "/***product_id_change(sale_order_line) - res ",res
            return res
        if product and (not qty) and (lot_org_id):
            res['value']['hd_lot_org'] = False
            res['warning'] = {'title':_('ATTENTION'), 'message': '%s' % _('You must enter a quantity')}
           #print "/***product_id_change(sale_order_line) - res ",res
            return res
        #hdcm 2014_01_06 
        product_saisi = self.pool.get('product.product').browse(cr, uid, product)
        res['value']['hd_track_lot'] = product_saisi.hd_track_lot
        #/hdcm 2014_01_06 
        if lot_org_id:#faire le controle du lot
            context['cas_so_normal_so_lines'] = True
            context['product_id'] = product
            #hdcm  2013_07_25 context['shop_id'] = shop_id
            context.update(CAS_SO_NORMAL)
            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, [lot_org_id], qty, uom, context=context)
            if verif_lots['infos'] != '': 
                res['value']['hd_lot_org'] = False
                #hdcm 2013_08_14 res['value']['price_unit'] = 0.0
                res['warning']={'title':_('ATTENTION'), 'message': verif_lots['infos']}
            else:
                res['value']['type'] = 'make_to_stock'
        '''product_saisi = self.pool.get('product.product').browse(cr, uid, product)
        context = {'cas_so_normal_so_lines': True, 'product_id': product, 'shop_id': shop_id}
        context.update(CAS_SO_NORMAL)
        res['domain']['hd_lot_org'] = [('id','in',self.pool.get('stock.production.lot').lot_control2(cr, uid, context=context)['lots_ok'])]
        res['domain']['product_uom'] = [('id','in',self.pool.get('product.uom').search(cr, uid, [('category_id', '=', product_saisi.uom_id.category_id.id)]))]'''
        #print "/***product_id_change(sale_order_line) - res ",res
        return res

    def lot_org_change_so_normal(self, cr, uid, ids, lot_org, product_qty, uom, product, shop_id, context=None):
       #print " ***lot_org_change_so_normal(sale_order_line)  ", ids, " - lot_org ", lot_org, " - product_qty ", product_qty, " - uom ", uom, " - product ", product, " - shop_id ", shop_id
        if context is None: context = {}
        if not product:
            res = {'value': {'hd_lot_org': False,'product_uom_qty': False, 'product_uom':False, 'name': False, 'price_unit': 0.0, 'discount': 0.0}}
           #print "/***lot_org_change_so_normal(sale_order_line) - res ", res
            return res
        res = {'value': {},'warning': {}}
        context = {'cas_so_normal_so_lines': True, 'product_id': product, 'shop_id': shop_id}
        context.update(CAS_SO_NORMAL)
        verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, [lot_org], product_qty, uom, context=context)
        if verif_lots['infos'] != '': 
            res['value']['hd_lot_org'] = False
            res['warning']={'title':_('ATTENTION'), 'message': verif_lots['infos']}
        else:
            res['value']['type'] = 'make_to_stock'
       #print "/***lot_org_change_so_normal(sale_order_line) - res ", res
        return res

    def button_confirm(self, cr, uid, ids, context=None):
        #print " ***button_confirm(sale.order.line) - ids ", ids, " - context ", context
        if context is None: context = {}
        message = ''
        lot_obj = self.pool.get('stock.production.lot')
        #    control of the lot information while confirming a sale, if lot changed since sale order entry
        for sale_line in self.browse(cr, uid, ids):
            if sale_line.hd_lot_org:
                context.update(CAS_SO_NORMAL)
                context['cas_so_normal_so_lines'] = sale_line.id
                context['product_id'] = sale_line.product_id.id
                context['shop_id'] = sale_line.order_id.shop_id.id
                verif_lots = lot_obj.lot_control2(cr, uid, [sale_line.hd_lot_org.id], sale_line.product_uom_qty, sale_line.product_uom.id,context=context)
                if verif_lots['infos'] != '': 
                    message += verif_lots['infos'] + "\n**************************************************************************************\n"
        if message != '':
            raise osv.except_osv(_('Information regarding the lot has changed'), _(message))
        #print "/***button_confirm(sale.order.line)"
        return self.write(cr, uid, ids, {'state':'confirmed'})    

sale_order_line()

