# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import netsvc
import decimal_precision as dp
from tools.translate import _
from osv import fields, osv
import time

class create_inventory_lot(osv.osv_memory):
    _name = 'create.inventory.lot'
    _description = 'Create lot by inventory'
    
    _columns = {
        'product_id': fields.many2one('product.product', 'Product', required=True, domain=[('hd_track_lot', '!=', False), ('active', '!=', False)]),
        'product_uom': fields.many2one('product.uom', 'Product UOM'),
        'location_id': fields.many2one('stock.location', 'Location', required=True),
        'qty': fields.float('Quantity', digits_compute=dp.get_precision('Qty Module Process'), required=True, help="Quantity in the lot."),
    }

    _defaults = {
        'qty': lambda *args: 0,
    }

    def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None:
            context = {}
        super(create_inventory_lot, self).view_init(cr, uid, fields_list, context=context)
        if len(context.get('active_ids',[])) > 1:
            raise osv.except_osv(_('Error!'), _('You cannot perform this operation on more than one Stock Inventories.'))
        if context.get('active_id', False):
            inventory_lot = self.pool.get('stock.inventory.lot').browse(cr, uid, context.get('active_id', False))
            if inventory_lot.state == 'done':
                raise osv.except_osv(_('Warning!'), _('Stock Inventory Lot is already Validated.'))
        return True

    def onchange_product_id(self, cr, uid, ids, product_id, context = None):
        #print " ***onchange_product_id(create_inventory_lot) - ids ", ids, " - product_id ", product_id
        if context is None: context = {}
        res = {}
        if not product_id:
            res = {'value': {'product_uom': False, 'location_id': False,'qty': 0.0}}
            #print "/***onchange_product_id(create_inventory_lot) - res ", res
            return res
        product = self.pool.get('product.product').browse(cr, uid, product_id)
        res = {'value': { 'product_uom': product.uom_id.id,'qty': 0.0}}
        #print "/***onchange_product_id(create_inventory_lot) - res ", res
        return res

    def onchange_product_uom(self, cr, uid, ids, product_id):
        #print "***on_change_product_uom(create_inventory_lot) - ids ",ids
        result = {}
        warning = {}
        if product_id:
            product = self.pool.get('product.product').browse(cr, uid, product_id)
            result['product_uom'] = product.uom_id.id
            warning={'title':_('Information'), 'message':_('You can not edit this field')}
        #print "/***on_change_product_uom(create_inventory_lot) - result ",result
        return {'value':result, 'warning':warning}


    def action_create_inventory_lot(self, cr, uid, ids, context=None):
        """ Create lot by inventory.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: List of IDs selected
        @param context: A standard dictionary
        @return: A dictionary .
        """
        #print " ***action_create_inventory_lot(create_inventory_lot) - ids ", ids, " - context ", context
        if context is None: context = {}
        inventory_lot_id = context['active_id']
        context['prog'] = 'action_create_inventory_lot(create_inventory_lot)'
        data = self.browse(cr, uid, ids[0], context=context)
        inventory_obj = self.pool.get('stock.inventory.lot')
        if inventory_obj.browse(cr, uid, inventory_lot_id).state == 'done':
            raise osv.except_osv(_('Invalid action !'), _('This inventory is %s.')%(inventory_obj.browse(cr, uid, inventory_lot_id).state))
        if data.qty <= 0.0:# si la quantite saisie <= 0
            raise osv.except_osv(_('Invalid action !'),_('The quantity entered must be positive !'))
        inventory_line_obj = self.pool.get('stock.inventory.lot.line')
        picking_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        product_obj = self.pool.get('product.product')
        inventory_name = self.pool.get('stock.inventory.lot').browse(cr, uid, inventory_lot_id).name
        # création d'un picking de type 'in'
        picking_id = picking_obj.create(cr, uid, {
                'origin': 'INV(create lot):'+ inventory_name,
                'type': 'in',
                'inventory_lot_id': inventory_lot_id,
                'move_type': 'one',#HDcm 2012_02_20 v506_06chem 
                })
        location_inventory_id = product_obj.browse(cr, uid,data.product_id.id).product_tmpl_id.property_stock_inventory.id
        # création du move avec location_id et location_dest_id pointant sur la zone inventaire de ce produit
        move_id = move_obj.create(cr, uid, {
                'name':'INV(create lot):'+inventory_name,
                'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
                'picking_id':picking_id,
                'product_id': data.product_id.id,
                'product_qty': data.qty,
                'product_uom': data.product_uom.id,
                'location_id': location_inventory_id,
                'location_dest_id': location_inventory_id,
                'state': 'draft',
                })
        wf_service = netsvc.LocalService("workflow")
        wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
        #HDcm 2012_04_04 picking_obj.force_assign(cr, uid,[picking_id])
        context['appel_prog'] = True#HDcm 2012_04_04
        picking_obj.force_assign(cr, uid,[picking_id], context=context)#HDcm 2012_04_04
        picking_obj.action_move(cr, uid, [picking_id])#HDcm 2012_02_20
        wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_done', cr)
        #print "/création d'un picking de type 'in'"
        # creer un split a partir du lot qui vient d'etre cree
        picking_split_id = picking_obj.create(cr, uid, {
                'origin': 'INV(create lot):'+ inventory_name,
                'type': 'split',
                'inventory_lot_id': inventory_lot_id,
                'move_type': 'one',#HDcm 2012_02_20 v506_06chem 
                })    
        # création du move avec location_id pointant sur la zone inventaire de ce produit
        move_split_id = move_obj.create(cr, uid, {
                'name':'INV(create lot):'+inventory_name,
                'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
                'picking_id':picking_split_id,
                'product_id': data.product_id.id,
                'product_qty': data.qty,
                'product_uom': data.product_uom.id,
                'hd_lot_org':move_obj.browse(cr, uid, move_id).hd_prodlot_id.id,
                'location_id': location_inventory_id,
                'location_dest_id': data.location_id.id,
                #HDcm 2012_02_20 v506_06chem 'state': 'assigned',
                'state': 'draft',
            })
        wf_service = netsvc.LocalService("workflow")
        wf_service.trg_validate(uid, 'stock.picking', picking_split_id, 'button_confirm', cr)
        #HDcm 2012_04_04 picking_obj.force_assign(cr, uid,[picking_split_id])
        context['appel_prog'] = True#HDcm 2012_04_04
        picking_obj.force_assign(cr, uid,[picking_split_id], context=context)#HDcm 2012_04_04 
        picking_obj.action_move(cr, uid, [picking_split_id])#HDcm 2012_02_20
        wf_service.trg_validate(uid, 'stock.picking', picking_split_id, 'button_done', cr)
        #print "/creer un split a partir du lot qui vient d'etre cree"
        # ajouter une ligne dans la liste de l'inventaire
        inventory_line={'lot_id': move_obj.browse(cr, uid, move_split_id).hd_prodlot_id.id, 
                        'inventory_id':inventory_lot_id,
                        'location_id':data.location_id.id,
                        'product_id':data.product_id.id,
                        'product_uom':data.product_uom.id,
                        'product_qty':data.qty,
                        'product_qty_before_inv':data.qty}#hdcm 2013_02_20
        inventory_line_obj.create(cr, uid, inventory_line)
        #print "/***action_create_inventory_lot(create_inventory_lot)"
        return {'type': 'ir.actions.act_window_close'}

create_inventory_lot()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

