# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import netsvc
import decimal_precision as dp
from tools.translate import _
from osv import fields, osv
import time

class make_multi_splitting_lot(osv.osv_memory):
    _name = 'make.multi.splitting.lot'
    _description = 'Make Multi Splitting Lot'
    
    def onchange_product_id(self, cr, uid, ids, product_id, context = None):
        """ On Change of Product ID getting the value of related Product and UoM.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of IDs selected 
         @param prod_id: Changed ID of Product 
         @return: A dictionary which gives the Product and the UoM of the changed Lot 
        """
        #print " ***onchange_product_id(make_multi_splitting_lot) - ids ", ids, " - product_id ", product_id
        if context is None: context = {}
        if not product_id:
            res = {'value': {'lot_id': False,'info': '', 'qty_taken': 0,'nb_lot': 0}}
            #print "/***onchange_product_id(make_multi_splitting_lot) - res ", res
            return res
        res = {'value': { 'qty_taken': 0.0}}
        #print "/***onchange_product_id(make_multi_splitting_lot) - res ", res
        return res

    def onchange_lot_id(self, cr, uid, ids, lot_id):
        """ On Change of Lot ID .
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of IDs selected 
         @param prod_id: Changed ID of Product 
         @return: A dictionary which gives the Product and the UoM of the changed Lot 
        """
        #print " ***onchange_lot_id(make_multi_splitting_lot) - ids ", ids, " - lot_id ", lot_id
        if lot_id:
            lot = self.pool.get('stock.production.lot').browse(cr, uid, lot_id)
            text_info = "Product : %s / Qty available : %.6f %s  " % (lot.product_id.name, lot.hd_qty_virt, lot.hd_product_uom_id.name)
            res = {'value': {'info': text_info}}
        else:
            res = {'value': {'info': ""}}
        #print "/***onchange_lot_id(make_multi_splitting_lot) - res ", res
        return res

    def onchange_qty_taken(self, cr, uid, ids, qty_taken):
        """ On Change of qty_taken .
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of IDs selected 'nb_lot':
         @param qty_taken: qte prelevee 
         @return: A dictionary which gives the number of Lot 
        """
        #print " ***onchange_qty_taken(make_multi_splitting_lot) - ids ", ids, " - lot_id ", lot_id
        if qty_taken:
            res = {'value': {'nb_lot': round(qty_taken)}}
        else:
            res = {'value': {'nb_lot': 0}}
        #print "/***onchange_qty_taken(make_multi_splitting_lot) - res ", res
        return res


    _columns = {
        'product_id': fields.many2one('product.product', 'Product', required=True),
        'lot_id': fields.many2one('stock.production.lot', 'Lot', required=True),
        'qty_taken': fields.integer('Quantity taken', required=True
                                  , help="Quantity taken in the lot."),
        'info': fields.char('Info', size=128),
        'nb_lot': fields.integer('Number of lots', required=True
                                  , help="Number of lots created with the quantity taken in the lot."),
    }

    _defaults = {
        'qty_taken': lambda *args: 0,
        'nb_lot': lambda *args: 0
    }

    def make_multi_splitting(self, cr, uid, ids, context=None):
        """ Creates multi splitting.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: List of IDs selected
        @param context: A standard dictionary
        @return: A dictionary .
        """
        #print " ***make_multi_splitting(make_multi_splitting_lot) - ids ", ids, " - context ", context
        if context is None: context = {}
        context['prog'] = 'make_multi_splitting(make_multi_splitting_lot)'
        data = self.browse(cr, uid, ids[0], context=context)
        lot_obj = self.pool.get('stock.production.lot')
        lot = lot_obj.browse(cr, uid, data.lot_id.id, context=context)
        seq_obj = self.pool.get('ir.sequence')
        wf_service = netsvc.LocalService("workflow")
        # si la quantite saisie > 0
        if data.qty_taken <= 0.0:
            raise osv.except_osv(_('Invalid action !'),_('The quantity entered must be positive !'))
        # si la quantite saisie > a la quantite virt du lot, message d'erreur
        if data.qty_taken > lot.hd_qty_virt:
            raise osv.except_osv(_('Invalid action !'),_('The amount entered is superior to the available quantity !'))
        # si la quantite saisie > a la quantite virt du lot, message d'erreur
        if data.nb_lot <= 0:
            raise osv.except_osv(_('Invalid action !'),_('The number of lots created with the quantity taken in the lot must be superior to zero!'))
        # creer le split
        pick_name = seq_obj.get(cr, uid, 'stock.picking.' + 'split')
        text_origin = "MULTI-SPLIT[lot %s]" % (lot.id,)
        split_picking_id = self.pool.get('stock.picking').create(cr, uid, {
            'name': pick_name,#HDcm 2012_02_29 v603_01chem
            'origin': text_origin,
            'type': 'split',
            'move_type': 'one',
            'state': 'draft',
            'auto_picking': False,
            'company_id': lot.company_id.id or 1,
        })
        for i in range(data.nb_lot):
            # création du move pour ce picking
            move_id = self.pool.get('stock.move').create(cr, uid, {
                'name':text_origin,
                'picking_id':split_picking_id,
                'product_id': lot.product_id.id,
                'product_qty': float(data.qty_taken) / data.nb_lot,
                'hd_lot_org': lot.id,
                'product_uom': lot.hd_product_uom_id.id,
                'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
                'location_id': lot.hd_location_id.id,
                'location_dest_id': lot.hd_location_id.id,
                'state': 'draft',
                'company_id': lot.company_id.id or 1,
            })
        wf_service.trg_validate(uid, 'stock.picking', split_picking_id, 'button_confirm', cr)
        self.pool.get('stock.picking').action_assign(cr, uid, [split_picking_id], context=context)
        #HDcm 2012_05_08  cette ligne est mise en commentaire pour que le picking ne soit pas validé automatiquement
        #wf_service.trg_validate(uid, 'stock.picking', split_picking_id, 'button_done', cr)
        
        view_obj = self.pool.get('ir.ui.view')
        view_id = view_obj.search(cr, uid, [('model', '=', 'stock.picking'), \
                                     ('name', '=', 'stock.splitting.form')])        
        #print "/***make_multi_splitting(make_multi_splitting_lot)"
        return {
                'name': _('Multi Splitting Lot'),
                'res_model': 'stock.picking',
                'res_id': int(split_picking_id),
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': view_id or False,
                'context': context,
                'type': 'ir.actions.act_window',
                }

make_multi_splitting_lot()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

