# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _

class stock_fill_inventory_lot(osv.osv_memory):
    _name = "stock.fill.inventory.lot"
    _description = "Import Inventory Lots"
    _columns = {
        'location_id': fields.many2one('stock.location', 'Location', required=True),
        'recursive': fields.boolean("Include children",help="If checked, lots contained in child locations of selected location will be included as well."),
    }
    def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None:
            context = {}
        super(stock_fill_inventory_lot, self).view_init(cr, uid, fields_list, context=context)
        if len(context.get('active_ids',[])) > 1:
            raise osv.except_osv(_('Error!'), _('You cannot perform this operation on more than one Stock Inventories.'))
        if context.get('active_id', False):
            inventory_lot = self.pool.get('stock.inventory.lot').browse(cr, uid, context.get('active_id', False))
            if inventory_lot.state == 'done':
                raise osv.except_osv(_('Warning!'), _('Stock Inventory Lot is already Validated.'))
        return True

    def fill_inventory(self, cr, uid, ids, context=None):
        """ To Import stock inventory according to products available in the selected locations.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: the ID or list of IDs if we want more than one
        @param context: A standard dictionary
        @return:
        """
        #print " ***fill_inventory(stock_fill_inventory_lot) - ids ", ids, " - context ", context
        if context is None: context = {}
        inventory_lot_id = context['active_id']
        context['prog'] = 'make_multi_splitting(make_multi_splitting_lot)'
        data = self.browse(cr, uid, ids[0], context=context)
        inventory_obj = self.pool.get('stock.inventory.lot')
        if inventory_obj.browse(cr, uid, inventory_lot_id).state == 'done':
            raise osv.except_osv(_('Invalid action !'), _('This inventory is %s.')%(inventory_obj.browse(cr, uid, inventory_lot_id).state))
        inventory_line_obj = self.pool.get('stock.inventory.lot.line')
        location_obj = self.pool.get('stock.location')
        lot_obj = self.pool.get('stock.production.lot')
        res={}
        res_location={}
        location_ids=[]
        if data.recursive :
            location_ids = location_obj.search(cr, uid, [('location_id', 'child_of', [data.location_id.id])])
        else:
            location_ids.append(data.location_id.id)
        lot_ids = lot_obj.search(cr, uid, [('hd_location_id','in',location_ids),('hd_qty_real', '>', 0.0)])
        for lot in lot_obj.browse(cr, uid, lot_ids):
            # verifier que cette ligne n'est pas deja dans la liste
            if not (len(inventory_line_obj.search(cr,uid,[('inventory_id','=',inventory_lot_id),('lot_id','=',lot.id)]))==0):
                continue
            inventory_line={'lot_id': lot.id, 'inventory_id':inventory_lot_id,'location_id':lot.hd_location_id.id,'product_id':lot.product_id.id,
                            'product_uom':lot.hd_product_uom_id.id,'product_qty_before_inv':lot.hd_qty_real,'product_qty':lot.hd_qty_real}
            inventory_line_obj.create(cr, uid, inventory_line)
        if(len(lot_ids)==0):
            raise osv.except_osv(_('Message !'), _('No lot in this location.'))
        return {'type': 'ir.actions.act_window_close'}
stock_fill_inventory_lot()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
